package edu.uoc.pfc.j2ee.jalonsod.mtp.config;

import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Method;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.logging.Logger;

import javax.xml.XMLConstants;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import edu.uoc.pfc.j2ee.jalonsod.mtp.MTPUtils;
import edu.uoc.pfc.j2ee.jalonsod.mtp.helper.ModelMapper;
import edu.uoc.pfc.j2ee.jalonsod.mtp.helper.ViewMapper;
import edu.uoc.pfc.j2ee.jalonsod.mtp.request.RequestData;

/**
 * File: Configurator.java
 * Created: 2012-04-21
 * 
 * Main object for configuration of the MTP Framework. Works as a singleton instance.
 * Usage:
 * <code>  Get singleton (first time loads with default config file):
 * 		Configurator.getInstance()
 *   Load configuration with user provided config file & obtain singleton):
 *   	Configurator.getInstance(String configFile)
 *   Use getters to obtain configuration objects.
 *   main method is for testing purposes</code>
 * <p>
 * The class has the inner class <code>SimpleErrorHandler</code>. This is an error handler for
 * loading documents.
 * 
 * @author Jos Alonso de Motta
 */
public class Configurator {
	private final static Logger logger = Logger.getLogger(RequestData.MTP_LOGGER);

	/**
	 * Error handler for loading documents.
	 */
	public class SimpleErrorHandler implements ErrorHandler {
	    public void warning(SAXParseException e) throws SAXException {
	    	logger.warning(e.getMessage());
	    }

	    public void error(SAXParseException e) throws SAXException {
	    	logger.severe(e.getMessage());
	    }

	    public void fatalError(SAXParseException e) throws SAXException {
	    	logger.severe(e.getMessage());
	    }
	}
	
	// Elements & attributes from XML configuration file
	public final static String TAG_ROOT_ELEMENT = "MTPConfig";
	public final static String TAG_ROOT_ATTR_PACKAGE_MODEL = "packageModel";
	public final static String TAG_ROOT_ATTR_SECURITY_ERROR_VIEW = "securityErrorView";
	public final static String TAG_ROOT_ATTR_VALIDATION_ERROR_VIEW = "validationErrorView";
	public final static String TAG_ROOT_ATTR_DEFAULT_ERROR_VIEW = "defaultErrorView";
	public final static String TAG_ROOT_ATTR_DEFAULT_VIEW = "defaultView";
	public final static String TAG_ROOT_ATTR_DEFAULT_MODEL = "defaultModel";
	public final static String TAG_ROOT_ATTR_DEFAULT_LANGUAGE = "defaultLanguage";
	public final static String TAG_ROOT_ATTR_MODEL_MAPPER_CLASS = "modelMapperClass";
	public final static String TAG_ROOT_ATTR_VIEW_MAPPER_CLASS = "viewMapperClass";
	public final static String TAG_ROOT_ATTR_VIEW_DEFAULT_DATE_FORMAT = "defaultDateFormat";
	public final static String TAG_FORM_ELEMENT = "form";
	public final static String TAG_FORM_ATTR_NAME = "name";
	public final static String TAG_FORM_PARAMETER_ELEMENT = "parameter";
	public final static String TAG_FORM_PARAMETER_ATTR_NAME = "name";
	public final static String TAG_FORM_PARAMETER_ATTR_DATATYPE = "datatype";
	public final static String TAG_FORM_PARAMETER_ATTR_REQUIRED = "required";
	public final static String TAG_FORM_PARAMETER_ATTR_VALIDATE_METHOD = "validateMethod";
	public final static String TAG_FORM_PARAMETER_ATTR_COPY_TO_MODEL = "copyToModel";
	public final static String TAG_FORM_PARAMETER_ATTR_MODEL_ATTRIBUTE = "modelAttribute";
	public final static String TAG_FORM_PARAMETER_ATTR_DEFAULT_VALUE = "defaultValue";
	public final static String TAG_FORM_PARAMETER_ATTR_DATE_FORMAT = "dateFormat";
	public final static String TAG_LANGUAGE_ELEMENT = "language";
	public final static String TAG_LANGUAGE_ATTR_NAME = "name";
	public final static String TAG_LANGUAGE_ATTR_LONG_NAME = "longName";
	public final static String TAG_LANGUAGE_ATTR_MESSAGE_FILE = "messageFile";
	public final static String TAG_MODEL_ELEMENT = "model";
	public final static String TAG_MODEL_ATTR_NAME = "name";
	public final static String TAG_MODEL_ATTR_CLASS_NAME = "className";
	public final static String TAG_MODEL_ATTR_FORM = "form";
	public final static String TAG_MODEL_ATTR_RUN_METHOD = "runMethod";
	public final static String TAG_MODEL_ATTR_SEND_REQUEST_DATA = "sendRequestData";
	public final static String TAG_MODEL_ATTR_VALIDATE_METHOD = "validateMethod";
	public final static String TAG_MODEL_ATTR_RESULT_NO_VALIDATE = "resultNoValidate";
	public final static String TAG_MODEL_ATTR_RESULT_SECURITY_ERROR = "resultSecurityError";
	public final static String TAG_MODEL_ATTR_DEFAULT_RESULT = "defaultResult";
	public final static String TAG_MODEL_GRANTEDROLE_ELEMENT = "grantedRole";
	public final static String TAG_MODEL_GRANTEDROLE_ATTR_NAME = "name";
	public final static String TAG_MODEL_RESULT_ELEMENT = "result";
	public final static String TAG_MODEL_RESULT_ATTR_NAME = "name";
	public final static String TAG_MODEL_RESULT_ATTR_NEXT_VIEW = "nextView";
	public final static String TAG_ROLE_ELEMENT = "role";
	public final static String TAG_ROLE_ATTR_NAME = "name";
	public final static String TAG_ROLE_ATTR_DEFAULT_ROLE = "defaultRole";
	public final static String TAG_VALIDATION_ELEMENT = "validation";
	public final static String TAG_VALIDATION_ATTR_NAME = "name";
	public final static String TAG_VALIDATION_ATTR_CLASSNAME = "className";
	public final static String TAG_VALIDATION_ATTR_METHODNAME = "methodName";
	public final static String TAG_VALIDATION_ATTR_DATATYPE = "datatype";

	// Datatypes of parameters from XML configuration file
	public final static String DATATYPE_STRING = "String";
	public final static String DATATYPE_BOOLEAN = "boolean";
	public final static String DATATYPE_INTEGER = "integer";
	public final static String DATATYPE_DOUBLE = "double";
	public final static String DATATYPE_DATE = "Date";

	// Schema for validating config file
	public final static String SCHEMA_FILE = "MTPConfig.xsd";

	// Config file loaded, with default value
	private static String configFile = "MTPConfig.xml";

	// Singleton attribute
	private static Configurator instance;
	
	// Attributes of main Configurator
	/**
	 * Name of the default package for storing Model objects.
	 */
	private String packageModel;

	/**
	 * Framework redirects to this view when a security error is thrown and model has no related.
	 * security error view 
	 */
	private String defaultSecurityErrorView;
	
	/**
	 * Framework redirects to this view when a validation error is thrown and model has no related.
	 * validations error view
	 */
	private String defaultValidationErrorView;
	
	/**
	 * Framework redirects to this view when an exception is thrown to the Controller during the
	 * request process.
	 */
	private String defaultErrorView;
	
	/**
	 * Framework redirects to this view when cannot map to any view.
	 */
	private String defaultView;
	
	/**
	 * Framework redirects to this model when cannot map to any model.
	 */
	private Model defaultModel;
	
	/**
	 * Default language for application user interface. Framework use this when no language is set and
	 * request supported languages does not match with framework supported languages.
	 */
	private Language defaultLanguage;
	
	/**
	 * Class used for mapping models. It can be framework supplied mapper or user supplied. It must
	 * implement <code>modelMapper</code> interface.
	 */
	private String modelMapperClass;
	
	/**
	 * A reference to the <code>modelMapper</code> class.
	 */
	private Class<ModelMapper> modelMapperClassReference;
	
	/**
	 * Class used for mapping views. It can be framework supplied mapper or user supplied. It must
	 * implement ViewMapper interface.
	 */
	private String viewMapperClass;
	
	/**
	 * A reference to the <code>viewMapper</code> class.
	 */
	private Class<ViewMapper> viewMapperClassReference;
	
	/**
	 * Default date format mask for date parameters. This will be used if parameters has no value on
	 * dateFormat attribute.
	 */
	private String defaultDateFormat;
	
	/**
	 * Forms supported by application.
	 */
	private Hashtable<String,Form> forms;
	
	/**
	 * Languages supported by application.
	 */
	private Hashtable<String,Language> languages;
	
	/**
	 * Models implemented by application.
	 */
	private Hashtable<String,Model> models;
	
	/**
	 * Security roles of application.
	 */
	private Hashtable<String,Role> roles;
	
	/**
	 * Field validations implemented by application.
	 */
	private Hashtable<String,Validation> validations;

	
	/**
	 * Constructor must be called only from singleton creation method getInstance.
	 * 
	 * @throws ConfiguratorException throws when failed to load configuration.
	 */
	private Configurator() throws ConfiguratorException{
		loadConfig();
	}

	
	// Methods for loading configuration
	/**
	 * Main method for loading configuration.
	 * 
	 * @throws ConfiguratorException throws when failed to load configuration.
	 */
	private void loadConfig() throws ConfiguratorException{
		logger.entering(MTPUtils.getClassName(), MTPUtils.getMethodName());
		resetAttributes();
		Document doc = validateConfigFile();
		// Load attributes with configuration file contents
		if(doc!=null){
			try {
				logger.fine("Reading config file");
				Node rootElement = doc.getFirstChild();
				NamedNodeMap nnm = rootElement.getAttributes();
				// Package model must be set before models
				setPackageModel(getAttributeValue(nnm, Configurator.TAG_ROOT_ATTR_PACKAGE_MODEL));
				setDefaultDateFormat(getAttributeValue(nnm, Configurator.TAG_ROOT_ATTR_VIEW_DEFAULT_DATE_FORMAT));
				// Load child lists
				loadList(doc.getElementsByTagName(Configurator.TAG_VALIDATION_ELEMENT), this.getClass().getDeclaredMethod("addValidationElement", Node.class));
				loadList(doc.getElementsByTagName(Configurator.TAG_LANGUAGE_ELEMENT), this.getClass().getDeclaredMethod("addLanguageElement", Node.class));
				loadList(doc.getElementsByTagName(Configurator.TAG_FORM_ELEMENT), this.getClass().getDeclaredMethod("addFormElement", Node.class));
				loadList(doc.getElementsByTagName(Configurator.TAG_ROLE_ELEMENT), this.getClass().getDeclaredMethod("addRoleElement", Node.class));
				loadList(doc.getElementsByTagName(Configurator.TAG_MODEL_ELEMENT), this.getClass().getDeclaredMethod("addModelElement", Node.class));
				// load attributes from root element
				setDefaultSecurityErrorView(getAttributeValue(nnm, Configurator.TAG_ROOT_ATTR_SECURITY_ERROR_VIEW));
				setDefaultValidationErrorView(getAttributeValue(nnm, Configurator.TAG_ROOT_ATTR_VALIDATION_ERROR_VIEW));
				setDefaultErrorView(getAttributeValue(nnm, Configurator.TAG_ROOT_ATTR_DEFAULT_ERROR_VIEW));
				setDefaultView(getAttributeValue(nnm, Configurator.TAG_ROOT_ATTR_DEFAULT_VIEW));
				setDefaultModel(getModelByKey(getAttributeValue(nnm, Configurator.TAG_ROOT_ATTR_DEFAULT_MODEL)));
				setDefaultLanguage(getLanguageByKey(getAttributeValue(nnm, Configurator.TAG_ROOT_ATTR_DEFAULT_LANGUAGE)));
				setModelMapperClass(getAttributeValue(nnm, Configurator.TAG_ROOT_ATTR_MODEL_MAPPER_CLASS));
				setViewMapperClass(getAttributeValue(nnm, Configurator.TAG_ROOT_ATTR_VIEW_MAPPER_CLASS));
				logger.fine("Config file has read successfully");
			} catch (SecurityException e) {
				logger.severe("Security exception while loading configuration!");
				e.printStackTrace();
				throw new ConfiguratorException("SecurityException while loading config file", e);
			} catch (NoSuchMethodException e) {
				logger.severe("NoSuchMethod exception while loading configuration!");
				e.printStackTrace();
				throw new ConfiguratorException("NoSuchMethodException while loading config file", e);
			}
		}else{
			logger.severe("Configuration file not loaded or validated!");
			throw new ConfiguratorException("Config file not read");
		}
		logger.fine("Configurator loaded: " + this.toString());
		logger.exiting(MTPUtils.getClassName(), MTPUtils.getMethodName());
	}

	/**
	 * Validate and load the config file.
	 * 
	 * @return the XML config file in a document object.
	 * @throws ConfiguratorException when an error loading or validating the XML schema or the XML file.  
	 */
	private Document validateConfigFile() throws ConfiguratorException{
		logger.entering(MTPUtils.getClassName(), MTPUtils.getMethodName());
		Document doc;
		// Validate & read configuration file
		try {
			
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			dbf.setValidating(false);
			dbf.setNamespaceAware(true);
			logger.fine("Loading schema for config file");
			// Schema file is located on the same package of this class.
			InputStream schemaInputStream = this.getClass().getResourceAsStream(Configurator.SCHEMA_FILE);
			SchemaFactory sf = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
			Schema schema = sf.newSchema(new StreamSource(schemaInputStream));
			javax.xml.validation.Validator validator = schema.newValidator();
			logger.fine("Validating config file");
			// Configuration files must be located in the root of class loader (WEB-INF/classes)
			InputStream configFileInputStreamValidate = this.getClass().getResourceAsStream("/" + configFile);
			Source xmlFile = new StreamSource(configFileInputStreamValidate);
			validator.validate(xmlFile);
			logger.fine("Loading config file");
			dbf.setSchema(schema);
			DocumentBuilder builder = dbf.newDocumentBuilder();
			builder.setErrorHandler(new SimpleErrorHandler());
			InputStream configFileInputStreamRead = this.getClass().getResourceAsStream("/" + configFile);
			doc = builder.parse(new InputSource(configFileInputStreamRead));
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
			throw new ConfiguratorException("ParserConfigurationException while validating config File", e);
		} catch (SAXException e){
			e.printStackTrace();
			throw new ConfiguratorException("SAXException while validating config File", e);
		} catch (IOException e) {
			e.printStackTrace();
			throw new ConfiguratorException("IOException while validating config File", e);
		} finally {
			
		}
		logger.exiting(MTPUtils.getClassName(), MTPUtils.getMethodName());
		return doc;
	}

	/**
	 * Initialize attribute values, prior to load configuration values.
	 */
	private void resetAttributes(){
		packageModel = null;
		defaultSecurityErrorView = null;
		defaultValidationErrorView = null;
		defaultModel = null;
		modelMapperClass = null;
		setModels(new Hashtable<String, Model>());
		setValidations(new Hashtable<String, Validation>());
		setLanguages(new Hashtable<String, Language>());
		setForms(new Hashtable<String, Form>());
		setRoles(new Hashtable<String, Role>());
	}
	
	/**
	 * Walks through a list of nodes and call supplied method of this object type for appropriate
	 * loading.
	 * 
	 * @param nl         List of nodes to load.
	 * @param loadMethod the method of this object to call for loading each node.
	 * @throws ConfiguratorException throws when load method throws any exception or when a there
	 *                               is a problem invoking method.
	 */
	private void loadList(NodeList nl, Method loadMethod) throws ConfiguratorException{
		try {
			for(int i = 0; i< nl.getLength(); i++){
				Node n = nl.item(i);
				loadMethod.invoke(this, n);
			}
		} catch (Exception e) {
			throw new ConfiguratorException("Exception while loading elements", e);
		}
	}
	
	/**
	 * Loads a form and its parameters from supplied node & child nodes.
	 * 
	 * @param n Node to load, it must be a Form.
	 * @throws ConfiguratorException throws when there is a problem creating parameter object.
	 */
	@SuppressWarnings("unused")		// Called with reflection API
	private void addFormElement(Node n) throws ConfiguratorException{
		NamedNodeMap nnm = n.getAttributes();
		String key = getAttributeValue(nnm, Configurator.TAG_FORM_ATTR_NAME);
		NodeList nl = n.getChildNodes();
		Hashtable<String, Parameter> htParameter = new Hashtable<String, Parameter>(nl.getLength()); 
		for(int i = 0; i < nl.getLength(); i++){
			if(nl.item(i).getNodeName().equals(Configurator.TAG_FORM_PARAMETER_ELEMENT)){
				NamedNodeMap nnmParameter = nl.item(i).getAttributes();
				String keyParameter = getAttributeValue(nnmParameter, Configurator.TAG_FORM_PARAMETER_ATTR_NAME);
				String dateFormat = getAttributeValue(nnmParameter, Configurator.TAG_FORM_PARAMETER_ATTR_DATE_FORMAT);
				if(dateFormat==null){
					dateFormat = getDefaultDateFormat();
				}
				Parameter parameter = Parameter.createParameter(keyParameter,
						getAttributeValue(nnmParameter, Configurator.TAG_FORM_PARAMETER_ATTR_DATATYPE),
						Boolean.parseBoolean(getAttributeValue(nnmParameter, Configurator.TAG_FORM_PARAMETER_ATTR_REQUIRED)),
						getValidationByKey(getAttributeValue(nnmParameter, Configurator.TAG_FORM_PARAMETER_ATTR_VALIDATE_METHOD)),
						Boolean.parseBoolean(getAttributeValue(nnmParameter, Configurator.TAG_FORM_PARAMETER_ATTR_COPY_TO_MODEL)),
						getAttributeValue(nnmParameter, Configurator.TAG_FORM_PARAMETER_ATTR_MODEL_ATTRIBUTE),
						getAttributeValue(nnmParameter, Configurator.TAG_FORM_PARAMETER_ATTR_DEFAULT_VALUE),
						dateFormat);
				htParameter.put(keyParameter, parameter);
			}
		}
		Form form = new Form(key, htParameter);
		getForms().put(key, form);
	}
	
	/**
	 * Loads a language from supplied node.
	 * 
	 * @param n Node to load, it must be a Language.
	 * @throws ConfiguratorException throws when there is a problem creating the language.
	 */
	@SuppressWarnings("unused")		// Called with reflection API
	private void addLanguageElement(Node n) throws ConfiguratorException{
		NamedNodeMap nnm = n.getAttributes();
		String key = getAttributeValue(nnm, Configurator.TAG_LANGUAGE_ATTR_NAME);
		Language v = new Language(key,
				getAttributeValue(nnm, Configurator.TAG_LANGUAGE_ATTR_LONG_NAME),
				getAttributeValue(nnm, Configurator.TAG_LANGUAGE_ATTR_MESSAGE_FILE));
		getLanguages().put(key, v);
	}
	
	/**
	 * Loads a model, its role-grants and results from supplied node & child nodes.
	 * 
	 * @param n Node to load, it must be a Language.
	 * @throws ConfiguratorException throws when there is a problem loading model and subordinate elements.
	 */
	@SuppressWarnings("unused")		// Called with reflection API
	private void addModelElement(Node n) throws ConfiguratorException{
		NamedNodeMap nnm = n.getAttributes();
		String key = getAttributeValue(nnm, Configurator.TAG_MODEL_ATTR_NAME);
		NodeList nl = n.getChildNodes();
		Hashtable<String, Role> htRole = new Hashtable<String, Role>(nl.getLength());
		Hashtable<String, Result> htResult = new Hashtable<String, Result>(nl.getLength());
		for(int i = 0; i < nl.getLength(); i++){
			if(nl.item(i).getNodeName().equals(Configurator.TAG_MODEL_GRANTEDROLE_ELEMENT)){
				NamedNodeMap nnmRole = nl.item(i).getAttributes();
				String keyRole = getAttributeValue(nnmRole, Configurator.TAG_MODEL_GRANTEDROLE_ATTR_NAME);
				Role role = getRoleByKey(keyRole);
				if(role==null) throw new ConfiguratorException("Role "+ keyRole + " not found in method "+ key);
				htRole.put(keyRole, role);
			} else if(nl.item(i).getNodeName().equals(Configurator.TAG_MODEL_RESULT_ELEMENT)){
				NamedNodeMap nnmResult = nl.item(i).getAttributes();
				String keyResult = getAttributeValue(nnmResult, Configurator.TAG_MODEL_RESULT_ATTR_NAME);
				Result result = new Result(keyResult,
						getAttributeValue(nnmResult, Configurator.TAG_MODEL_RESULT_ATTR_NEXT_VIEW));
				if(htResult.get(keyResult)!=null) throw new ConfiguratorException("Result " + keyResult + " duplicated on model "+ key);
				htResult.put(keyResult, result);
			}
		}
		Form form = getFormByKey(getAttributeValue(nnm, Configurator.TAG_MODEL_ATTR_FORM));
		String strResultNoValidate = getAttributeValue(nnm, Configurator.TAG_MODEL_ATTR_RESULT_NO_VALIDATE);
		Result resultNoValidate = null;
		if(strResultNoValidate!=null) resultNoValidate = htResult.get(strResultNoValidate);
		String strResultSecurityError = getAttributeValue(nnm, Configurator.TAG_MODEL_ATTR_RESULT_SECURITY_ERROR);
		Result resultSecurityError = null;
		if(strResultSecurityError!=null) resultSecurityError = htResult.get(strResultSecurityError);
		String strDefaultResult = getAttributeValue(nnm, Configurator.TAG_MODEL_ATTR_DEFAULT_RESULT);
		Result defaultResult = htResult.get(strDefaultResult);
		if(defaultResult==null) throw new ConfiguratorException("DefaultResult "+ strDefaultResult + " for model " + key + " does not exists");
		String className = getAttributeValue(nnm, Configurator.TAG_MODEL_ATTR_CLASS_NAME);
		if(className.substring(0, 1).equals(".")) className = getPackageModel() + className; 
		Model model = new Model(key,
				className,
				form,
				getAttributeValue(nnm, Configurator.TAG_MODEL_ATTR_RUN_METHOD),
				Boolean.parseBoolean(getAttributeValue(nnm, Configurator.TAG_MODEL_ATTR_SEND_REQUEST_DATA)),
				getAttributeValue(nnm, Configurator.TAG_MODEL_ATTR_VALIDATE_METHOD),
				htResult.get(getAttributeValue(nnm, Configurator.TAG_MODEL_ATTR_DEFAULT_RESULT)),
				resultNoValidate,
				resultSecurityError,
				htResult,
				htRole
				);
		getModels().put(key, model);
	}
	
	/**
	 * Loads a role from supplied node.
	 * 
	 * @param n Node to load, it must be a Role.
	 * @throws ConfiguratorException throws when there is a problem creating the role object.
	 */
	@SuppressWarnings("unused")		// Called with reflection API
	private void addRoleElement(Node n) throws ConfiguratorException {
		NamedNodeMap nnm = n.getAttributes();
		String key = getAttributeValue(nnm, Configurator.TAG_ROLE_ATTR_NAME);
		Role r = new Role(key,
				Boolean.parseBoolean(getAttributeValue(nnm, Configurator.TAG_ROLE_ATTR_DEFAULT_ROLE)));
		getRoles().put(key, r);
	}
	
	/**
	 * Loads a validation from supplied node.
	 * 
	 * @param n Node to load, it must be a Validation.
	 * @throws ConfiguratorException throws when there is a problem creating the validation object.
	 */
	@SuppressWarnings("unused")		// Called with reflection API
	private void addValidationElement(Node n) throws ConfiguratorException {
		NamedNodeMap nnm = n.getAttributes();
		String key = getAttributeValue(nnm, Configurator.TAG_VALIDATION_ATTR_NAME);
		Validation v = new Validation(key,
				getAttributeValue(nnm, Configurator.TAG_VALIDATION_ATTR_CLASSNAME),
				getAttributeValue(nnm, Configurator.TAG_VALIDATION_ATTR_METHODNAME),
				getAttributeValue(nnm, Configurator.TAG_VALIDATION_ATTR_DATATYPE));
		getValidations().put(key, v);
	}
	
	/**
	 * Obtains the value of the supplied attribute from the nodemap.
	 * 
	 * @param nnm      the <code>map</code> with attributes of a node.
	 * @param attrName the name of the attribute to get.
	 * @return         the value, or <code>null</code> if attribute is empty or not present.
	 */
	private String getAttributeValue(NamedNodeMap nnm, String attrName){
		String value="";
		Node ni = nnm.getNamedItem(attrName);
		if(ni!=null) value = ni.getNodeValue();
		if(value.equals("")) value = null;
		return value;
	}

	
	// getters & setters
	/**
	 * Returns the singleton (also creates & loads configuration the first time).
	 * 
	 * @return the <b>singleton</b> instance.
	 * @throws ConfiguratorException throws when failed to load configuration.
	 */
	public static Configurator getInstance() throws ConfiguratorException {
		if(instance == null) {
			logger.fine("Creating singleton");
			instance = new Configurator();
		}
		return instance;
	}
	
	/**
	 * Returns the singleton loaded with configFile. It reloads singleton when previous configuration exists.
	 * 
	 * @param configFile the configuration file name for loading the configuration.
	 * @return           the <code>singleton</code> instance
	 * @throws ConfiguratorException throws when there is a problem loading the configuration.
	 */
	public static Configurator getInstance(String configFile) throws ConfiguratorException{
		setConfigFile(configFile);  // Change the config file
		Configurator i;
		if(instance == null) {
			// Instance is not loaded, so getInstance & it will auto-load
			i = getInstance();
		} else {
			// Instance is already loaded, so, getInstance & load again with supplied config file
			i = getInstance();
			i.loadConfig();
		}
		return i;
	}
	
	public Hashtable<String, Form> getForms() {
		return forms;
	}
	
	public void setForms(Hashtable<String, Form> forms) {
		this.forms = forms;
	}
	
	public Hashtable<String, Language> getLanguages() {
		return languages;
	}
	
	public void setLanguages(Hashtable<String, Language> languages) {
		this.languages = languages;
	}
	
	public Hashtable<String, Model> getModels() {
		return models;
	}
	
	public void setModels(Hashtable<String, Model> models) {
		this.models = models;
	}
	
	public Hashtable<String, Role> getRoles() {
		return roles;
	}
	
	public void setRoles(Hashtable<String, Role> roles) {
		this.roles = roles;
	}
	
	public Hashtable<String, Validation> getValidations() {
		return validations;
	}
	
	public void setValidations(Hashtable<String, Validation> validations) {
		this.validations = validations;
	}
	
	public static String getConfigFile() {
		return configFile;
	}
	
	// Private because config file must be supplied with singleton method
	private static void setConfigFile(String configFile) {
		logger.fine("Config file changed to " + configFile);
		Configurator.configFile = configFile;
	}
	
	public String getPackageModel() {
		return packageModel;
	}
	
	public void setPackageModel(String packageModel) {
		this.packageModel = packageModel;
	}
	
	public String getDefaultSecurityErrorView() {
		return defaultSecurityErrorView;
	}
	
	public void setDefaultSecurityErrorView(String defaultSecurityErrorView) {
		this.defaultSecurityErrorView = defaultSecurityErrorView;
	}
	
	public String getDefaultValidationErrorView() {
		return defaultValidationErrorView;
	}
	
	public void setDefaultValidationErrorView(String defaultValidationErrorView) {
		this.defaultValidationErrorView = defaultValidationErrorView;
	}
	
	public String getDefaultErrorView() {
		return defaultErrorView;
	}
	
	public void setDefaultErrorView(String defaultErrorView) {
		this.defaultErrorView = defaultErrorView;
	}
	
	public String getDefaultView() {
		return defaultView;
	}
	
	public void setDefaultView(String defaultView) {
		this.defaultView = defaultView;
	}
	
	public Model getDefaultModel() {
		return defaultModel;
	}
	
	public void setDefaultModel(Model defaultModel) {
		this.defaultModel = defaultModel;
	}
	
	public Language getDefaultLanguage() {
		return defaultLanguage;
	}
	
	public void setDefaultLanguage(Language defaultLanguage) {
		logger.fine("Default langugage changed to " + defaultLanguage.getName());
		this.defaultLanguage = defaultLanguage;
	}
	
	public String getModelMapperClass() {
		return modelMapperClass;
	}
	
	@SuppressWarnings("unchecked")
	public void setModelMapperClass(String modelMapperClass) throws ConfiguratorException {
		Class<ModelMapper> c = null;
		try {
			c = (Class<ModelMapper>)Class.forName(modelMapperClass);
		} catch (ClassNotFoundException e) {
			throw new ConfiguratorException("ModelMapperClass " + modelMapperClass + " does not exist", e);
		}
		this.modelMapperClass = modelMapperClass;
		this.modelMapperClassReference = c;
	}
	
	public Class<ModelMapper> getModelMapperClassReference() {
		return modelMapperClassReference;
	}
	
	public String getViewMapperClass() {
		return viewMapperClass;
	}
	
	@SuppressWarnings("unchecked")
	public void setViewMapperClass(String viewMapperClass) throws ConfiguratorException{
		Class<ViewMapper> c = null;
		try {
			c = (Class<ViewMapper>)Class.forName(viewMapperClass);
		} catch (ClassNotFoundException e) {
			throw new ConfiguratorException("ViewMapperClass " + viewMapperClass + " does not exist", e);
		}
		this.viewMapperClass = viewMapperClass;
		this.viewMapperClassReference = c;
	}
	
	public Class<ViewMapper> getViewMapperClassReference() {
		return viewMapperClassReference;
	}

	public String getDefaultDateFormat() {
		return defaultDateFormat;
	}

	public void setDefaultDateFormat(String defaultDateFormat) {
		this.defaultDateFormat = defaultDateFormat;
	}


	// These methods search child elements by key
	public Form getFormByKey(String key){
		if(key==null) return null;
		if(forms!=null) return forms.get(key);
		else return null;
	}
	
	public Language getLanguageByKey(String key){
		if(key==null) return null;
		if(languages!=null) return languages.get(key);
		else return null;
	}
	
	public Model getModelByKey(String key){
		if(key==null) return null;
		if(models!=null) return models.get(key);
		else return null;
	}
	
	public Role getRoleByKey(String key){
		if(key==null) return null;
		if(roles!=null) return roles.get(key);
		else return null;
	}
	
	public Validation getValidationByKey(String key){
		if(key==null) return null;
		if(validations!=null) return validations.get(key);
		else return null;
	}
	
	
	/**
	 * Get roles marked as default.
	 * 
	 * @return a new list with the roles marked as default. 
	 */
	public Hashtable<String, Role> getDefaultRoles(){
		Hashtable<String, Role> defaultRoles = new Hashtable<String, Role>();
		Enumeration<Role> roleEnumerator = getRoles().elements();
		while(roleEnumerator.hasMoreElements()){
			Role r = roleEnumerator.nextElement();
			if(r.getDefaultRole()){
				defaultRoles.put(r.getName(), r);
			}
		}
		return defaultRoles;
	}

	@Override
	public String toString() {
		return "Configurator [packageModel=" + packageModel
				+ ", defaultSecurityErrorView=" + defaultSecurityErrorView
				+ ", defaultValidationErrorView=" + defaultValidationErrorView
				+ ", defaultErrorView=" + defaultErrorView
				+ ", defaultView=" + defaultView
				+ ", defaultModel=" + defaultModel
				+ ", defaultLanguage=" + defaultLanguage
				+ ", modelMapperClass=" + modelMapperClass
				+ ", modelMapperClassReference=" + modelMapperClassReference
				+ ", viewMapperClass=" + viewMapperClass
				+ ", viewMapperClassReference=" + viewMapperClassReference
				+ ", defaultDateFormat=" + defaultDateFormat
				+ ", forms=" + forms
				+ ", languages=" + languages
				+ ", models=" + models
				+ ", roles=" + roles
				+ ", validations=" + validations
				+ "]";
	}

	
	
	/**
	 * main method for testing purposes.
	 * 
	 * @param args
	 */
	public static void main(String args[]) {
		try {
			System.out.println("Testing Configurator...");
			Configurator i = Configurator.getInstance();

			// Check Configurator
			System.out.println("\nCONFIGURATOR");
			System.out.println(i.toString());

		} catch (ConfiguratorException e) {
			e.printStackTrace();
		}
	}
}
