package edu.uoc.pfc.j2ee.jalonsod.mtp.config;

import java.lang.reflect.Method;
import java.util.Hashtable;

import edu.uoc.pfc.j2ee.jalonsod.mtp.request.RequestData;

/**
 * File: Model.java
 * Created: 2012-04-22
 * 
 * Contains the model metadata needed for framework to handle appropriately the application model objects.
 * A model represents a class from user application for linking presentation and business tiers.
 * It integrates application code & data to process a kind of request from user.
 * 
 * @author Jos Alonso de Motta
 */
public class Model {
	private String name;
	private String className;
	private Class<java.lang.Object> classReference;
	private Form form;						// Optional
	private boolean sendRequestData;
	private String runMethod;
	private Method runMethodReference;
	private String validateMethod;			// Optional
	private Method validateMethodReference;	// Optional
	private Result defaultResult;
	private Result resultNoValidate;		// Optional
	private Result resultSecurityError;		// Optional
	private Hashtable<String, Result> results;		// Cannot be empty
	private Hashtable<String, Role> grantedRoles;	// Can be empty

	public Model(String name, String className, Form form, String runMethod,
			boolean sendRequestData, String validateMethod, Result defaultResult,
			Result resultNoValidate, Result resultSecurityError,
			Hashtable<String, Result> results, Hashtable<String, Role> grantedRoles) throws ConfiguratorException{
		if(name==null||name.equals("")) throw new ConfiguratorException("Model element has null or empty name");
		if(className==null||className.equals("")) throw new ConfiguratorException("Model element " + name + " has null or empty className");
		if(runMethod==null||runMethod.equals("")) throw new ConfiguratorException("Model element " + name + " has null or empty runMethod");
		if(defaultResult==null) throw new ConfiguratorException("Model element " + name + " has null defaultResult");
		if(results==null||results.isEmpty()) throw new ConfiguratorException("Model element " + name + " has null or empty results");
		if(grantedRoles==null) throw new ConfiguratorException("Model element " + name + " has null grantedRoles");
		setName(name);
		setClassName(className);
		setForm(form);
		setSendRequestData(sendRequestData);
		setRunMethod(runMethod);
		setValidateMethod(validateMethod);
		setDefaultResult(defaultResult);
		setResultNoValidate(resultNoValidate);
		setResultSecurityError(resultSecurityError);
		setResults(results);
		setGrantedRoles(grantedRoles);
	}
	
	// getters & setters
	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}
	public String getClassName() {
		return className;
	}
	@SuppressWarnings("unchecked")
	public void setClassName(String className) throws ConfiguratorException {
		Class<java.lang.Object> c = null;
		try {
			c = (Class<java.lang.Object>)Class.forName(className);
		} catch (ClassNotFoundException e) {
			throw new ConfiguratorException("Model element " + name + " has nonexisting classname " + className, e);
		}
		this.className = className;
		setClassReference(c);
		setRunMethodReference(null);
		setValidateMethodReference(null);
	}
	public Class<java.lang.Object> getClassReference() {
		return classReference;
	}
	// Private because class reference is updated from setClassName
	private void setClassReference(Class<java.lang.Object> classReference) {
		this.classReference = classReference;
	}
	public Form getForm() {
		return form;
	}
	public void setForm(Form form) {
		this.form = form;
	}
	public String getRunMethod() {
		return runMethod;
	}
	public void setRunMethod(String runMethod) throws ConfiguratorException{
		Method r = null;
		if(!sendRequestData){
			try {
				r = getClassReference().getMethod(runMethod);
				@SuppressWarnings("rawtypes")
				Class returnType = r.getReturnType(); 
				if(returnType!=String.class)
					throw new ConfiguratorException("Model element " + getName() + " has run method " + getClassName() + "." + runMethod + "() whit return type " + returnType + ", but it must return a String");
			} catch (SecurityException e) {
				throw new ConfiguratorException("Model element " + getName() + " has SecurityException with run method " + getClassName() + "." + runMethod + "()", e);
			} catch (NoSuchMethodException e) {
				throw new ConfiguratorException("Model element " + getName() + " does not have run method " + getClassName() + "." + runMethod + "()", e);
			}
		}else{
			try {
				r = getClassReference().getMethod(runMethod, RequestData.class);
				@SuppressWarnings("rawtypes")
				Class returnType = r.getReturnType(); 
				if(returnType!=String.class)
					throw new ConfiguratorException("Model element " + getName() + " has run method " + getClassName() + "." + runMethod + "(Request) whit return type " + returnType + ", but it must return a String");
			} catch (SecurityException e) {
				throw new ConfiguratorException("Model element " + getName() + " has SecurityException with run method " + getClassName() + "." + runMethod + "(RequestData)", e);
			} catch (NoSuchMethodException e) {
				throw new ConfiguratorException("Model element " + getName() + " does not have run method " + getClassName() + "." + runMethod + "(RequestData)", e);
			}
		}
		this.runMethod = runMethod;
		setRunMethodReference(r);
	}
	public Method getRunMethodReference() {
		return runMethodReference;
	}
	// Private because run method reference is updated locally
	private void setRunMethodReference(Method runMethodReference) {
		this.runMethodReference = runMethodReference;
	}
	public boolean getSendRequestData() {
		return sendRequestData;
	}
	public void setSendRequestData(boolean sendRequestData) {
		this.sendRequestData = sendRequestData;
	}
	public String getValidateMethod() {
		return validateMethod;
	}
	public void setValidateMethod(String validateMethod) throws ConfiguratorException {
		Method v = null;
		if(validateMethod!=null){
			try {
				v  = getClassReference().getMethod(validateMethod);
				@SuppressWarnings("rawtypes")
				Class returnType = v.getReturnType();
				if(returnType!=boolean.class)
					throw new ConfiguratorException("Model element " + getName() + " has validate method " + getClassName() + "." + validateMethod + "() whit return type " + returnType + ", but it must return a boolean");
			} catch (SecurityException e) {
				throw new ConfiguratorException("Model element " + getName() + " has SecurityException with run method " + getClassName() + "." + validateMethod + "()", e);
			} catch (NoSuchMethodException e) {
				throw new ConfiguratorException("Model element " + getName() + " does not have run method " + getClassName() + "." + validateMethod + "()", e);
			}
		}
		this.validateMethod=validateMethod;
		setValidateMethodReference(v);
	}
	public Method getValidateMethodReference() {
		return validateMethodReference;
	}
	// Private because validate method reference is updated locally
	private void setValidateMethodReference(Method validateMethodReference) {
		this.validateMethodReference = validateMethodReference;
	}
	public Result getDefaultResult() {
		return defaultResult;
	}
	public void setDefaultResult(Result defaultResult) {
		this.defaultResult = defaultResult;
	}
	public Result getResultNoValidate() {
		return resultNoValidate;
	}
	public void setResultNoValidate(Result resultNoValidate) {
		this.resultNoValidate = resultNoValidate;
	}
	public Result getResultSecurityError() {
		return resultSecurityError;
	}
	public void setResultSecurityError(Result resultSecurityError) {
		this.resultSecurityError = resultSecurityError;
	}
	public Hashtable<String, Result> getResults() {
		return results;
	}
	public void setResults(Hashtable<String, Result> results) {
		this.results = results;
	}
	public Hashtable<String, Role> getGrantedRoles() {
		return grantedRoles;
	}
	public void setGrantedRoles(Hashtable<String, Role> roles) {
		this.grantedRoles = roles;
	}
	
	@Override
	public String toString() {
		return "Model [name=" + name
				+ ", className=" + className
				+ ", classReference=" + classReference
				+ ", form=" + form
				+ ", sendRequestData=" + sendRequestData
				+ ", runMethod=" + runMethod
				+ ", runMethodReference=" + runMethodReference
				+ ", validateMethod=" + validateMethod
				+ ", validateMethodReference=" + validateMethodReference
				+ ", defaultResult=" + defaultResult
				+ ", resultNoValidate=" + resultNoValidate
				+ ", resultSecurityError=" + resultSecurityError
				+ ", results=" + results
				+ ", grantedRoles=" + grantedRoles
				+ "]";
	}
}
