package edu.uoc.pfc.j2ee.jalonsod.mtp.config;

/**
 * File: Parameter.java
 * Created: 2012-04-22
 * 
 * Parameter configuration metadata.
 * Contains the metadata of a single parameter submitted by the user in a request.
 * Parameters in MTP are parts of a Form.
 * Parameter has a subtypes, one for each Java datatype.
 * To create new parameter objects of appropriate datatype, developers must call to factory method
 * createParameter. New datatypes must have a subclass and be taken into account in createParameter
 * factory method.
 * When a user request parameter is received , the value is loaded into a <code>ParameterInstance</code>. 
 * 
 * @author Jos Alonso de Motta
 * @see Form
 * @see ParameterInstance
 */
public abstract class Parameter {
	protected String name;
	protected String datatype;					// Value from Configurator.DATATYPE_*
	protected Object datatypeClassReference;
	protected boolean required;
	protected Validation validateMethod;		// Optional
	protected boolean copyToModel;
	protected String modelAttribute;			// Optional. If null or empty, copy from name

	/**
	 * Constructor of parameter. Developers must call to factory method createParameter to create
	 * new Parameter subclass instances
	 * 
	 * @throws ConfiguratorException Throws when received parameters are invalid
	 */
	protected Parameter(String name, String datatype, boolean required,
			Validation validateMethod, boolean copyToModel,
			String modelAttribute) throws ConfiguratorException{
		setName(name);
		setDatatype(datatype);
		setRequired(required);
		setValidateMethod(validateMethod);
		setCopyToModel(copyToModel);
		setModelAttribute(modelAttribute);
	}
	
	/**
	 * Factory method to create new Parameter instances
	 * 
	 * @param name           Name of parameter. Framework use this to locate user request parameter. Mandatory.
	 * @param datatype       Datatype of parameter (check Configurator.DATATYPE_* for supported datatypes). Mandatory.
	 * @param required       true if parameter value is mandatory. Mandatory.
	 * @param validateMethod Validate method. Optional.
	 * @param copyToModel    true if parameter must be replicated to model instance. Mandatory.
	 * @param modelAttribute The name of the attribute in the model class.
	 *                       Example: If modelAttribute is "keyValue" framework will call to modelInstance.setKeyValue(value)
	 *                       (first letter in uppercase). Optional. In case of null or empty will be populated with param name.
	 * @param defaultValue   The default value of the instances of this parameter. Optional.
	 * @param dateFormat     Date format mask. Mandatory for Date parameters. Otherwise it is ignored.
	 * @return               A new Parameter instance of appropriate subclass.
	 * @throws ConfiguratorException
	 */
	public static Parameter createParameter(String name, String datatype, boolean required,
			Validation validateMethod, boolean copyToModel,
			String modelAttribute, String defaultValue, String dateFormat) throws ConfiguratorException{
		if(datatype.equalsIgnoreCase(Configurator.DATATYPE_STRING)){
			return new StringParameter(name, datatype, required, validateMethod, copyToModel, modelAttribute, defaultValue);
		}else if (datatype.equalsIgnoreCase(Configurator.DATATYPE_BOOLEAN)){
			return new BooleanParameter(name, datatype, required, validateMethod, copyToModel, modelAttribute, defaultValue);
		}else if (datatype.equalsIgnoreCase(Configurator.DATATYPE_INTEGER)){
			return new IntegerParameter(name, datatype, required, validateMethod, copyToModel, modelAttribute, defaultValue);
		}else if (datatype.equalsIgnoreCase(Configurator.DATATYPE_DOUBLE)){
			return new DoubleParameter(name, datatype, required, validateMethod, copyToModel, modelAttribute, defaultValue);
		}else if (datatype.equalsIgnoreCase(Configurator.DATATYPE_DATE)){
			return new DateParameter(name, datatype, required, validateMethod, copyToModel, modelAttribute, dateFormat, defaultValue);
		}
		return null;
	}
	
	// getters & setters
	public String getName() {
		return name;
	}
	public void setName(String name) throws ConfiguratorException{
		if(name==null||name.equals("")) throw new ConfiguratorException("Parameter element has null or empty name");
		this.name = name;
	}
	public String getDatatype() {
		return datatype;
	}
	// Datatype cannot be changed. That's why this setter is private
	private void setDatatype(String datatype) throws ConfiguratorException {
		if(datatype==null||datatype.equals("")) throw new ConfiguratorException("Parameter element " + name + " has null or empty datatype");
		this.datatype = datatype;
	}
	public boolean getRequired() {
		return required;
	}
	public void setRequired(boolean required) {
		this.required = required;
	}
	public Validation getValidateMethod() {
		return validateMethod;
	}
	public void setValidateMethod(Validation validateMethod) throws ConfiguratorException {
		if(validateMethod!=null&&(!validateMethod.getDatatype().equals(datatype))) throw new ConfiguratorException("Parameter element " + name + " of type " + datatype + " has validation function of type " + validateMethod.getDatatype());
		this.validateMethod = validateMethod;
	}
	public boolean getCopyToModel() {
		return copyToModel;
	}
	public void setCopyToModel(boolean copyToModel) {
		this.copyToModel = copyToModel;
	}
	public String getModelAttribute() {
		return modelAttribute;
	}
	public void setModelAttribute(String modelAttribute) {
		if(modelAttribute==null || modelAttribute.equals("")){
			modelAttribute = name;			// Use name as the default model attribute
		}
		this.modelAttribute = modelAttribute;
	}
	public abstract Object getDefaultValue();
	public abstract void setDefaultValue(Object defaultValue);
	public abstract Class<?> getDatatypeReference();

	@Override
	public String toString() {
		return "Parameter [name=" + name
				+ ", datatype=" + datatype
				+ ", required=" + required
				+ ", validateMethod=" + validateMethod
				+ ", copyToModel=" + copyToModel
				+ ", modelAttribute=" + modelAttribute
				+ ", defaultValue=" + getDefaultValue()
				+ "]";
	}
}
