package edu.uoc.pfc.j2ee.jalonsod.mtp.helper;

import java.util.logging.Logger;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;

import edu.uoc.pfc.j2ee.jalonsod.mtp.MTPException;
import edu.uoc.pfc.j2ee.jalonsod.mtp.request.HttpRequestData;
import edu.uoc.pfc.j2ee.jalonsod.mtp.request.RequestData;
import edu.uoc.pfc.j2ee.jalonsod.mtp.request.RequestDataPrivate;

/**
 * File: ContextFactory.java
 * Created: 2012-04-28
 * 
 * Creates <code>requestData</code> object from a request. This factory creates <code>requestData</code>
 * according to request type. The session is created in this step if request has no previous session.
 * 
 * @author Jos Alonso de Motta
 * @see RequestData
 * @see RequestDataPrivate
 */
public class ContextFactory {
	private final static Logger logger = Logger.getLogger(RequestData.MTP_LOGGER);

	/**
	 * Main method for creating <code>requestData</code> context object based on the type of the
	 * request object. Only <code>HttpServletRequest</code> is implemented, but framework is open
	 * to new types.
	 * 
	 * @param request the request object received from the user.
	 * @return the MTP requestData of the current request.
	 * @throws MTPException throws when request is not of the expected types.
	 */
	public static RequestDataPrivate getContext(ServletRequest request) throws MTPException{
		if(HttpServletRequest.class.isInstance(request)){
			logger.fine("Request is of type HttpServletRequest.");
			return getHttpContext((HttpServletRequest)request);
		}else{
			logger.warning("Request is of unsupported type " + request.getClass().getName());
			throw new MTPException("Only requests of type " + HttpServletRequest.class.getName()
					+ " are implemented, but has received type " + request.getClass().getName());
		}
	}
	
	/**
	 * Creates <code>requestData</code> objects for <code>httpServletRequest</code> request type.
	 * 
	 * @param request the request object received from the user.
	 * @return the MTP requestData of the current request of <code>http</code> type.
	 * @throws MTPException throws when there is an error creating the requestData object.
	 * @see ContextFactory#getRequestData(Object)
	 */
	private static HttpRequestData getHttpContext(HttpServletRequest request) throws MTPException{
		HttpRequestData requestData = null;
		requestData = new HttpRequestData((HttpServletRequest)request);
		// Create session if necessarily
		if(!requestData.hasValidSession()){
			logger.fine("Creating a new session for request.");
			requestData.createSession();
		}
		return requestData;
	}

	/**
	 * Returns the requestData stored as an attribute. Request types are the same types as declared
	 * in <code>getContext</code>.
	 * 
	 * @param request the request object received from user.
	 * @return the MTP requestData of the current request.
	 * @see ContextFactory#getContext(ServletRequest)
	 */
	public static RequestData getRequestData(Object request){
		if(HttpServletRequest.class.isInstance(request)){
			return (RequestData)((HttpServletRequest)request).getAttribute(RequestData.REQUEST_REQUESTDATA_ATTRIBUTE);
		}else{
			logger.warning("Request is of unsupported type " + request.getClass().getName());
			return null;
		}
	}
}
