package edu.uoc.pfc.j2ee.jalonsod.mtp.helper;

import java.lang.reflect.Method;
import java.util.Enumeration;
import java.util.logging.Logger;

import edu.uoc.pfc.j2ee.jalonsod.mtp.MTPException;
import edu.uoc.pfc.j2ee.jalonsod.mtp.config.Form;
import edu.uoc.pfc.j2ee.jalonsod.mtp.config.Validation;
import edu.uoc.pfc.j2ee.jalonsod.mtp.request.FormInstance;
import edu.uoc.pfc.j2ee.jalonsod.mtp.request.ParameterInstance;
import edu.uoc.pfc.j2ee.jalonsod.mtp.request.RequestData;
import edu.uoc.pfc.j2ee.jalonsod.mtp.request.RequestDataPrivate;

/**
 * File: ContextFactory.java
 * Created: 2012-04-28
 * 
 * Loads and validates the request parameters into <code>formInstance</code> and model instance.
 * 
 * @author Jos Alonso de Motta
 */
public class Validator {
	private final static Logger logger = Logger.getLogger(RequestData.MTP_LOGGER);
	
	/**
	 * Loads and validates the request parameters. The validation is done in four stages:
	 * Stage 1: Converts request data to Java types.
	 * Stage 2: Validates that required fields are present.
	 * Stage 3: Validates using Validation functions for single fields.
	 * Before next stage, loads parameters into the model instance. 
	 * Stage 4: Validates entire model using validation function from model instance.
	 * It stops when a stage is finished and has any error.
	 * It records the validation state in the RequestData object.
	 * 
	 * @param requestData the MTP requestData of the current request.
	 */
	public void validateRequest(RequestDataPrivate requestData) {
		Form form = requestData.getModel().getForm();
		if(form==null){
			requestData.setValidationStatus(RequestDataPrivate.VALIDATION_OK);
			logger.fine("Model " + requestData.getModel().getName() + " has no form. No need to validate.");
			return;
		}
		FormInstance formInstance = new FormInstance(form);
		requestData.setFormInstance(formInstance);
		
		// Stage 1: Converts request data to Java types
		boolean stage1Successful = true; 
		Enumeration<ParameterInstance> enumeratorParameters = formInstance.getParameters().elements();
		while(enumeratorParameters.hasMoreElements()){
			ParameterInstance parameterInstance = enumeratorParameters.nextElement();
			String parameterName = parameterInstance.getParameter().getName();
			String parameterValue = requestData.getRequestParameter(parameterName);
			try {
				parameterInstance.setStringValue(parameterValue);
				parameterInstance.setValidationStatus(ParameterInstance.VALIDATION_NOT_VALIDATED);
			} catch (MTPException e) {
				parameterInstance.setValidationStatus(ParameterInstance.VALIDATION_DATATYPE_CONVERSION_ERROR);
				stage1Successful = false;
				logger.fine(e.getMessage() + " while converting form " + formInstance.getForm().getName() + ", parameter "
						+ parameterName + " with value " + parameterValue + " to datatype " + parameterInstance.getParameter().getDatatype());
			}
		}
		if(!stage1Successful){
			requestData.setValidationStatus(RequestDataPrivate.VALIDATION_ERROR_STAGE_1_DATATYPE_CONVERSION);
			return;
		}
		
		// Stage 2: Validates that required fields are present
		boolean stage2Successful = true;
		enumeratorParameters = formInstance.getParameters().elements();
		while(enumeratorParameters.hasMoreElements()){
			ParameterInstance parameterInstance = enumeratorParameters.nextElement();
			if(parameterInstance.getParameter().getRequired() && parameterInstance.getValue()==null){
				parameterInstance.setValidationStatus(ParameterInstance.VALIDATION_REQUIRED_VALUE_IS_NULL);
				logger.fine("Model " + requestData.getModel().getName() + " has required parameter " + parameterInstance.getParameter().getName() + " empty.");
				stage2Successful = false;
			}
		}
		if(!stage2Successful){
			requestData.setValidationStatus(RequestDataPrivate.VALIDATION_ERROR_STAGE_2_REQUIRED_FIELDS);
			return;
		}
		
		// Stage 3: Validates using Validation functions for single fields
		boolean stage3Successful = true;
		enumeratorParameters = formInstance.getParameters().elements();
		while(enumeratorParameters.hasMoreElements()){
			ParameterInstance parameterInstance = enumeratorParameters.nextElement();
			String parameterName = parameterInstance.getParameter().getName();
			String parameterValue = requestData.getRequestParameter(parameterName);
			Validation validation = parameterInstance.getParameter().getValidateMethod();
			if(validation!=null && parameterInstance.getValue()!=null){
				Method validationMethod = validation.getMethodReference();
				try {
					boolean validationResult = (Boolean)validationMethod.invoke(null, parameterInstance.getValue());
					if(validationResult){
						parameterInstance.setValidationStatus(ParameterInstance.VALIDATION_OK);
					}else{
						parameterInstance.setValidationStatus(ParameterInstance.VALIDATION_FUNCTION_ERROR);
						logger.fine("Model " + requestData.getModel().getName() + " has parameter " + parameterName + " which does not pass Validation " + validation.getName() + ".");
						stage3Successful = false;
					}
				} catch (Exception e) {
					parameterInstance.setValidationStatus(ParameterInstance.VALIDATION_FUNCTION_ERROR);
					stage3Successful = false;
					e.printStackTrace();
					logger.warning("Exception " + e.getMessage() + " while validating form " + formInstance.getForm().getName() + ", parameter "
							+ parameterName + " with value " + parameterValue + " and datatype " + parameterInstance.getParameter().getDatatype()
							+ " using method " + validation.getClassName() + "." + validation.getMethodName());
				}
			}else{
				parameterInstance.setValidationStatus(ParameterInstance.VALIDATION_OK);
			}
		}
		if(!stage3Successful){
			requestData.setValidationStatus(RequestDataPrivate.VALIDATION_ERROR_STAGE_3_FIELD_VALIDATION_FUNCTIONS);
			return;
		}

		// Set parameter values
		try {
			setParameterValues(requestData);
		} catch (MTPException e) {
			requestData.setValidationStatus(RequestDataPrivate.VALIDATION_ERROR_LOADING_PARAMETER_VALUES_INTO_MODEL);
			logger.warning("Exception " + e.getMessage() + " while loading parameter values into application model");
			e.printStackTrace();
			return;
		}
		
		// Stage 4: Validates using validation function for entire model
		boolean stage4Successful = true;
		Object modelInstance = requestData.getModelInstance();
		Method validationMethod = requestData.getModel().getValidateMethodReference();
		if(validationMethod!=null){
			try {
				boolean validationResult = (Boolean)validationMethod.invoke(modelInstance);
				if(!validationResult){
					logger.fine("Model " + requestData.getModel().getName() + " does not pass validation function " + requestData.getModel().getRunMethod() + ".");
					stage4Successful = false;
				}
			} catch (Exception e) {
				stage4Successful = false;
				e.printStackTrace();
				logger.warning("Exception " + e.getMessage() + " while loading parameters & validate form " + formInstance.getForm().getName()
						+ " with validating function " + requestData.getModel().getClassName()+ "." + requestData.getModel().getValidateMethod());
			}
		}
		if(!stage4Successful){
			requestData.setValidationStatus(RequestDataPrivate.VALIDATION_ERROR_STAGE_4_MODEL_VALIDATION_FUNCTION);
			return;
		}

		// Validation pass successful all stages.
		requestData.setValidationStatus(RequestDataPrivate.VALIDATION_OK);
	}

	/**
	 * Write the parameters from the request into the application modelInstance.
	 * 
	 * @param requestData the MTP requestData of the current request.
	 * @throws MTPException throws when there is a problem setting the values into the application model.
	 */
	public void setParameterValues(RequestDataPrivate requestData) throws MTPException {
		FormInstance formInstance = requestData.getFormInstance();
		if(formInstance!=null){
			Enumeration<ParameterInstance> enumeratorParameters = formInstance.getParameters().elements();
			Object modelInstance = requestData.getModelInstance();
			while(enumeratorParameters.hasMoreElements()){
				ParameterInstance parameterInstance = enumeratorParameters.nextElement();
				if(parameterInstance.getParameter().getCopyToModel()){
					Method method;
					String modelAttribute = parameterInstance.getParameter().getModelAttribute();
					String methodName = "set" + modelAttribute.substring(0, 1).toUpperCase() + modelAttribute.substring(1);
					try {
						method = requestData.getModel().getClassReference().getMethod(methodName, parameterInstance.getParameter().getDatatypeReference());
						method.invoke(modelInstance, parameterInstance.getValue());
					} catch (Exception e) {
						logger.warning("Exception " + e.getMessage() + " while setting value of parameter "
								+ parameterInstance.getParameter().getName() + " with value "
								+ parameterInstance.getValue() + " when calling method "
								+ requestData.getModel().getClassName()+ "." + methodName);
						e.printStackTrace();
						throw new MTPException("Exception while calling "
								+ requestData.getModel().getClassName()+ "." + requestData.getModel().getValidateMethod(), e);
					}
				}
			}
		}
	}
}
