package edu.uoc.pfc.j2ee.jalonsod.mtp.request;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import edu.uoc.pfc.j2ee.jalonsod.mtp.config.Configurator;
import edu.uoc.pfc.j2ee.jalonsod.mtp.config.ConfiguratorException;
import edu.uoc.pfc.j2ee.jalonsod.mtp.config.Language;
import edu.uoc.pfc.j2ee.jalonsod.mtp.config.LanguageEntry;
import edu.uoc.pfc.j2ee.jalonsod.mtp.config.Model;
import edu.uoc.pfc.j2ee.jalonsod.mtp.config.Result;
import edu.uoc.pfc.j2ee.jalonsod.mtp.config.Role;


/**
 * File: HttpRequestData.java
 * Created: 2012-04-30
 * 
 * HttpRequestData is the HTTP implementation of the RequestDataPrivate. Applications should not use
 * directly this class, they must use the <code>requestData</code> interface. 
 * 
 * @author Jos Alonso de Motta
 * @see    RequestData
 * @see    RequestDataPrivate
 */
public class HttpRequestData implements RequestDataPrivate {
	/**
	 * The request received from the user.
	 */
	private HttpServletRequest request;
	
	/**
	 * The framework configuration of the application.
	 */
	private Configurator configurator;
	
	/**
	 * The <code>language</code> inferred to the request. It's one of the languages from the
	 * <code>configurator</code>.
	 */
	private Language requestLanguage;
	
	/**
	 * The model configuraction selected for the request. Is stored during the
	 * <code>ModelMapper</code> stage of the request. It's one of the models from the
	 * <code>configurator</code>.
	 */
	private Model model;
	
	/**
	 * The <code>string</code> that the application model object run method has returned. It's value
	 * is stored after the <code>ModelRunner</code> stage of the request. It can be null.
	 */
	private String modelResponse;
	
	/**
	 * The result inferred to the request. It's value is stored during the <code>ViewMapper</code>
	 * stage of the request. It's one of the results from the <code>configurator</code> associated
	 * with the <code>model</code> selected for the request. It can be null.
	 */
	private Result result;
	
	/**
	 * The view selected to redirect the response (like a jsp page). It's value is stored during the
	 * <code>ViewMapper</code> stage of the request.
	 */
	private String view;
	
	/**
	 * The application model object. It's value is stored during the <code>ModelMapper</code> stage
	 * of the request. 
	 */
	private Object modelInstance;
	
	/**
	 * The <code>formInstance</code> object with the parameter values of the request associated with
	 * the <code>model</code>. It's value is stored during the <code>Validator</code> stage of the
	 * request. 
	 */
	private FormInstance formInstance;
	
	/**
	 * The authorization status of the request. The authorization is done during the
	 * <code>AccessControl</code> stage of the request. <code>false</code> means that request
	 * is not authorized, <code>true</code> means request is authorized to execute. 
	 */
	private boolean authorized;
	
	/**
	 * The validation status of the request. Possible values are defined by
	 * <code>RequestData.VALIDATION_*</code> constants. The request is full validated when the
	 * value is RequestData.VALIDATION_OK
	 */
	private int validationStatus;

	/**
	 * Constructor. Creates the object, sets initial values for request and infers the language.
	 * 
	 * @param request the request object received from the user. 
	 * @throws ConfiguratorException throws when the <code>configurator</code> has any problem.
	 */
	public HttpRequestData( HttpServletRequest request ) throws ConfiguratorException{
		setRequest(request);
		setConfigurator(Configurator.getInstance());
		setRequestLanguage(inferUserLanguage());
		setModel(null);
		setModelResponse(null);
		setResult(null);
		setView(null);
		setModelInstance(null);
		setFormInstance(null);
		authorize(false);
		setValidationStatus(VALIDATION_NO_VALIDATED);
		setRequestAttribute(REQUEST_REQUESTDATA_ATTRIBUTE, this);
	}

	@Override
	public Configurator getConfigurator() {
		return configurator;
	}
	
	private void setConfigurator(Configurator configurator){
		this.configurator = configurator;
	}
	
	@Override
	public Hashtable<String, Language> getLanguages(){
		return configurator.getLanguages();
	}

	@Override
	public void createSession() {
		getRequest().getSession();
		setDefaultRoles();
	}

	@Override
	public boolean hasValidSession() {
		HttpSession session = getSession(); 
		return (session!=null);
	}

	private HttpSession getSession() {
		return getRequest().getSession(false);
	}
	
	@Override
	public void logoutSession(){
		if(hasValidSession()) getSession().invalidate();
	}

	@Override
	public Object getSessionAttribute(String attributeName) {
		return getSession().getAttribute(attributeName);
	}

	@Override
	public void setSessionAttribute(String attributeName, Object attributeObject) {
		getSession().setAttribute(attributeName, attributeObject);
	}
	
	@Override
	public void deleteSessionAttribute(String attributeName){
		getSession().removeAttribute(attributeName);
	}

	@Override
	public Hashtable<String, Role> getRoles() {
		@SuppressWarnings("unchecked")
		Hashtable<String, Role> sessionRoles = (Hashtable<String, Role>)getSession().getAttribute(SESSION_ROLES_ATTRIBUTE);
		return sessionRoles;
	}

	@Override
	public void setRoles(Hashtable<String, Role> sessionRoles) {
		getSession().setAttribute(SESSION_ROLES_ATTRIBUTE, sessionRoles);
	}
	
	/**
	 * Stores the default roles defined by the <code>configurator</code> into the session.
	 */
	private void setDefaultRoles(){
		setRoles(configurator.getDefaultRoles());
	}

	@Override
	public Language getUserLanguage() {
		return requestLanguage;
	}

	@Override
	public void setUserLanguage(Language userLanguage) {
		getSession().setAttribute(SESSION_LANGUAGE_ATTRIBUTE, userLanguage);
		setRequestLanguage(userLanguage);
	}
	
	/**
	 * Sets the <code>language</code> of the request.
	 * 
	 * @param requestLanguage the <code>language</code> assigned to the request.
	 */
	private void setRequestLanguage(Language requestLanguage) {
		this.requestLanguage = requestLanguage;
	}
	
	/**
	 * Algorithm to infer the language of the request based on:
	 * <ul>
	 * <li>the language of the session if it was set on previous request.
	 * <li>the languages supported by the client agent.
	 * <li>the default language of application configuration.
	 * </ul>
	 * 
	 * @return the language inferred by the algorithm.
	 */
	private Language inferUserLanguage() {
		Language inferredLanguage = null;
		// 1. Try to obtain language from session (language has been set in previous request).
		if(hasValidSession()){
			inferredLanguage = (Language)getSession().getAttribute(SESSION_LANGUAGE_ATTRIBUTE);
		}
		// 2. Try to infer language from request
		if(inferredLanguage==null){
			Enumeration<Locale> locales = getRequest().getLocales();
			while(locales.hasMoreElements()){
				Locale locale = locales.nextElement();
				Language language = configurator.getLanguageByKey(locale.getLanguage());
				if(language!=null){
					inferredLanguage = language;
					break;
				}
			}
		}
		// 3. Use default language
		if(inferredLanguage==null){
			inferredLanguage = getConfigurator().getDefaultLanguage();
		}
		return inferredLanguage;
	}
	
	@Override
	public String getMessage(String key) {
		String message = null;
		LanguageEntry languageEntry = requestLanguage.getLanguageEntryByKey(key);
		if(languageEntry!=null){
			message = languageEntry.getValue();
		}
		return message;
	}


	private HttpServletRequest getRequest(){
		return request;
	}

	private void setRequest(HttpServletRequest request){
		this.request = request;
	}
	
	@Override
	public Object getRequestAttribute(String attributeName) {
		return getRequest().getAttribute(attributeName);
	}

	@Override
	public void setRequestAttribute(String attributeName, Object attributeObject) {
		getRequest().setAttribute(attributeName, attributeObject);
	}

	@Override
	public String getRequestParameter(String parameterName) {
		return getRequest().getParameter(parameterName);
	}
	
	@Override
	public Model getModel() {
		return model;
	}

	@Override
	public void setModel(Model model) {
		this.model = model;
	}

	@Override
	public Object getModelInstance() {
		return modelInstance;
	}

	@Override
	public void setModelInstance(Object modelInstance) {
		this.modelInstance = modelInstance;
		getRequest().setAttribute(REQUEST_MODEL_ATTRIBUTE, modelInstance);
	}

	@Override
	public FormInstance getFormInstance() {
		return formInstance;
	}

	@Override
	public void setFormInstance(FormInstance formInstance) {
		this.formInstance = formInstance;
	}

	@Override
	public String getModelResponse() {
		return modelResponse;
	}

	@Override
	public void setModelResponse(String modelResponse) {
		this.modelResponse = modelResponse;
	}

	@Override
	public Result getResult() {
		return result;
	}

	@Override
	public void setResult(Result result) {
		this.result = result;
	}

	@Override
	public String getView() {
		return view;
	}

	@Override
	public void setView(String view) {
		this.view = view;
	}

	@Override
	public boolean isAuthorized() {
		return authorized;
	}

	@Override
	public void authorize(boolean authorized) {
		this.authorized = authorized;
	}

	@Override
	public int getValidationStatus(){
		return validationStatus;
	}
	
	@Override
	public void setValidationStatus(int validationStatus){
		this.validationStatus = validationStatus;
	}

	@Override
	public boolean isValidated() {
		return (validationStatus == VALIDATION_OK);
	}
}
