-- Crear esquema
CREATE DATABASE ampa;

-- Crear usuario de conexión y dar permisos
CREATE USER ampauser;
GRANT ALL PRIVILEGES ON ampa.* TO ampauser IDENTIFIED BY 'ampapassword';

-- Cambiar al esquema AMPA para crear los objetos.
USE ampa;

-- Creación de tablas
CREATE TABLE SolicitudSocio
(usuario        VARCHAR(100) NOT NULL,
 password       VARCHAR(30) NOT NULL,
 direccion      VARCHAR(100) NOT NULL,
 fechaSolicitud TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
 CONSTRAINT pk_solSocio PRIMARY KEY(usuario)
);

CREATE TABLE SolicitudPadre
(idSolPadre INT NOT NULL AUTO_INCREMENT,
 usuario    VARCHAR(100) NOT NULL,
 nombre     VARCHAR(100) NOT NULL,
 apellidos  VARCHAR(100) NOT NULL,
 documento  VARCHAR(100) NOT NULL,
 email      VARCHAR(100),
 telefonos  VARCHAR(50),
 CONSTRAINT pk_solPadre PRIMARY KEY(idSolPadre),
 CONSTRAINT fk_solPadre_solSocio FOREIGN KEY (usuario) REFERENCES SolicitudSocio(usuario) ON DELETE CASCADE
);

CREATE TABLE SolicitudAlumno
(usuario         VARCHAR(100) NOT NULL,
 idSolAlumno     INT NOT NULL AUTO_INCREMENT,
 nombre          VARCHAR(100) NOT NULL,
 apellidos       VARCHAR(100) NOT NULL,
 curso           VARCHAR(10) NOT NULL,
 fechaNacimiento DATE NOT NULL,
 CONSTRAINT pk_solAlumno PRIMARY KEY(idSolAlumno),
 CONSTRAINT fk_solAlumno_solSocio FOREIGN KEY (usuario) REFERENCES SolicitudSocio(usuario) ON DELETE CASCADE
);

CREATE TABLE Usuario
(usuario         VARCHAR(100) NOT NULL,
 password        VARCHAR(24) NOT NULL,
 tipo            VARCHAR(1) NOT NULL ,
 CONSTRAINT pk_usuario PRIMARY KEY(usuario),
 CONSTRAINT chk_usuario_tipo CHECK (tipo IN ('A','S'))
);

CREATE TABLE Administrador
(usuario         VARCHAR(100) NOT NULL,
 CONSTRAINT pk_administrador PRIMARY KEY(usuario),
 CONSTRAINT fk_administrador_usuario FOREIGN KEY(usuario) REFERENCES Usuario(usuario) ON DELETE CASCADE
);

CREATE TABLE Socio
(usuario        VARCHAR(100) NOT NULL,
 direccion      VARCHAR(100) NOT NULL,
 fechaSolicitud TIMESTAMP NOT NULL,
 valido         VARCHAR(1) NOT NULL DEFAULT 'S',
 CONSTRAINT pk_socio PRIMARY KEY(usuario),
 CONSTRAINT fk_socio_usuario FOREIGN KEY(usuario) REFERENCES Usuario(usuario) ON DELETE CASCADE,
 CONSTRAINT chk_socio_valido CHECK (valido in ('S','N'))
);

CREATE TABLE Padre
(idPadre   INT NOT NULL AUTO_INCREMENT,
 usuario   VARCHAR(100) NOT NULL,
 nombre    VARCHAR(100) NOT NULL,
 apellidos VARCHAR(100) NOT NULL,
 documento VARCHAR(100) NOT NULL,
 email     VARCHAR(100),
 telefonos VARCHAR(50),
 CONSTRAINT pk_padre PRIMARY KEY(idPadre),
 CONSTRAINT fk_padre_socio FOREIGN KEY (usuario) REFERENCES Socio(usuario) ON DELETE CASCADE
);

CREATE TABLE Alumno
(idAlumno        INT NOT NULL AUTO_INCREMENT,
 usuario         VARCHAR(100) NOT NULL,
 nombre          VARCHAR(100) NOT NULL,
 apellidos       VARCHAR(100) NOT NULL,
 curso           VARCHAR(10) NOT NULL,
 fechaNacimiento DATE NOT NULL,
 CONSTRAINT pk_alumno PRIMARY KEY(idAlumno),
 CONSTRAINT fk_alumno_socio FOREIGN KEY (usuario) REFERENCES Socio(usuario) ON DELETE CASCADE
);

CREATE TABLE Actividad
(idActividad       INT NOT NULL AUTO_INCREMENT,
 nombre            VARCHAR(100) NOT NULL,
 descripcionLarga  VARCHAR(100) NOT NULL,
 requiereValidar   VARCHAR(1) NOT NULL,
 cuotaEstandar     DOUBLE NOT NULL,
 plazasDisponibles INTEGER NOT NULL,
 paraAlumnos       VARCHAR(1) NOT NULL,
 paraPadres        VARCHAR(1) NOT NULL,
 CONSTRAINT chk_actividad_validar CHECK (requiereValidar in ('S','N')),
 CONSTRAINT chk_actividad_paraAlumnos CHECK (paraAlumnos in ('S','N')),
 CONSTRAINT chk_actividad_paraPadres CHECK (paraPadres in ('S','N')),
 CONSTRAINT pk_actividad PRIMARY KEY(idActividad)
);

CREATE TABLE PeriodoActividad
(idPeriodo   INT NOT NULL AUTO_INCREMENT,
 idActividad INT NOT NULL,
 inicio      DATE NOT NULL,
 fin         DATE,
 cuota       DOUBLE,
 CONSTRAINT pk_periodoActividad PRIMARY KEY(idPeriodo),
 CONSTRAINT uk_periodoActividad UNIQUE KEY(idActividad,idPeriodo),
 CONSTRAINT fk_periodoActividad_actividad FOREIGN KEY(idActividad) REFERENCES Actividad(idActividad) ON DELETE CASCADE
);

CREATE TABLE PadreActividad
(idPadre     INT NOT NULL,
 idActividad INT NOT NULL,
 confirmado  VARCHAR(1),
 CONSTRAINT chk_padreActividad_confirmado CHECK (confirmado in ('S','N')),
 CONSTRAINT pk_padreActividad PRIMARY KEY(idPadre,idActividad),
 CONSTRAINT fk_padreActividad_actividad FOREIGN KEY(idActividad) REFERENCES Actividad(idActividad),
 CONSTRAINT fk_padreActividad_padre FOREIGN KEY(idPadre) REFERENCES Padre(idPadre) ON DELETE CASCADE
);

CREATE TABLE PadreActividadPeriodo
(idPadre     INT NOT NULL,
 idActividad INT NOT NULL,
 idPeriodo   INT NOT NULL,
 CONSTRAINT pk_padreActPer PRIMARY KEY(idPadre,idActividad,idPeriodo),
 CONSTRAINT fk_padreActPer_padreAct FOREIGN KEY(idPadre,idActividad) REFERENCES PadreActividad(idPadre,idActividad) ON DELETE CASCADE,
 CONSTRAINT fk_padreAct_periodoAct FOREIGN KEY(idActividad,idPeriodo) REFERENCES PeriodoActividad(idActividad,idPeriodo) ON DELETE CASCADE
);

CREATE TABLE AlumnoActividad
(idAlumno    INT NOT NULL,
 idActividad INT NOT NULL,
 confirmado  VARCHAR(1),
 CONSTRAINT chk_alumnoAct_conf CHECK (confirmado in ('S','N')),
 CONSTRAINT pk_alumnoActd PRIMARY KEY(idAlumno,idActividad),
 CONSTRAINT fk_alumnoAct_act FOREIGN KEY(idActividad) REFERENCES Actividad(idActividad),
 CONSTRAINT fk_alumnoAct_alumno FOREIGN KEY(idAlumno) REFERENCES Alumno(idAlumno) ON DELETE CASCADE
);

CREATE TABLE AlumnoActividadPeriodo
(idAlumno    INT NOT NULL,
 idActividad INT NOT NULL,
 idPeriodo   INT NOT NULL,
 CONSTRAINT pk_alumnoActPer PRIMARY KEY(idAlumno,idActividad,idPeriodo),
 CONSTRAINT fk_alumnoActPer_alumnoAct FOREIGN KEY(idAlumno,idActividad) REFERENCES AlumnoActividad(idAlumno,idActividad) ON DELETE CASCADE,
 CONSTRAINT fk_alumnoAct_periodoAct FOREIGN KEY(idActividad,idPeriodo) REFERENCES PeriodoActividad(idActividad,idPeriodo) ON DELETE CASCADE
);


-- Usuario administrador
insert into usuario(usuario,password,tipo) values('admin','manager','A');
insert into administrador(usuario) values('admin');
commit;


-- Carga de datos de prueba
insert into Actividad(idActividad,nombre,descripcionLarga,requiereValidar,cuotaEstandar,plazasDisponibles,paraAlumnos,paraPadres) values(1,'Desayuno','Servicio de desayuno','N',55,999,'S','N');
insert into PeriodoActividad(idPeriodo,idActividad,inicio,fin,cuota) values(1,1,'2012-09-10','2012-09-30',null);
insert into PeriodoActividad(idPeriodo,idActividad,inicio,fin,cuota) values(2,1,'2012-10-01','2012-10-31',null);
insert into PeriodoActividad(idPeriodo,idActividad,inicio,fin,cuota) values(3,1,'2012-11-01','2012-11-30',null);
insert into PeriodoActividad(idPeriodo,idActividad,inicio,fin,cuota) values(4,1,'2012-12-01','2012-12-30',null);
insert into PeriodoActividad(idPeriodo,idActividad,inicio,fin,cuota) values(5,1,'2013-01-01','2013-01-31',null);
insert into PeriodoActividad(idPeriodo,idActividad,inicio,fin,cuota) values(6,1,'2013-02-01','2013-02-28',null);
insert into PeriodoActividad(idPeriodo,idActividad,inicio,fin,cuota) values(7,1,'2013-03-01','2013-03-31',null);
insert into PeriodoActividad(idPeriodo,idActividad,inicio,fin,cuota) values(8,1,'2013-04-01','2013-04-30',null);
insert into PeriodoActividad(idPeriodo,idActividad,inicio,fin,cuota) values(9,1,'2013-05-01','2013-05-31',null);
insert into PeriodoActividad(idPeriodo,idActividad,inicio,fin,cuota) values(10,1,'2013-06-01','2013-06-22',40);

insert into Actividad(idActividad,nombre,descripcionLarga,requiereValidar,cuotaEstandar,plazasDisponibles,paraAlumnos,paraPadres) values(2,'Cambio','Servicio de cambios','N',35,999,'S','N');
insert into PeriodoActividad(idPeriodo,idActividad,inicio,fin,cuota) values(11,2,'2012-09-01','2013-06-30',null);

insert into Actividad(idActividad,nombre,descripcionLarga,requiereValidar,cuotaEstandar,plazasDisponibles,paraAlumnos,paraPadres) values(3,'Excursión','Excursión a Lagunas de Cabañeros','N',10,500,'S','S');
insert into PeriodoActividad(idPeriodo,idActividad,inicio,fin,cuota) values(12,3,'2012-06-28','2012-06-28',null);

insert into usuario(usuario,password,tipo) values('user@example.com','userpass','S');
insert into socio(usuario,direccion,fechaSolicitud) values('user@example.com','C/ Constelación, 5 - 28032 Madrid','2012-05-15');
insert into Padre(idPadre,usuario,nombre,apellidos,documento,email,telefonos) values(1,'user@example.com','Jose','Vega Bolinaga','00000000Z','jose@example.com','6000000');
insert into Padre(idPadre,usuario,nombre,apellidos,documento,email,telefonos) values(2,'user@example.com','Astrid','Molina','00000001Y','astrid@example.com','6000001');
insert into Alumno(idAlumno,usuario,nombre,apellidos,curso,fechaNacimiento) values(1,'user@example.com','Santiago','Vega Molina','3ºC','2003-03-17');
insert into Alumno(idAlumno,usuario,nombre,apellidos,curso,fechaNacimiento) values(2,'user@example.com','Laura','Vega Molina','3ºC','2003-03-17');

commit;
