package edu.uoc.pfc.j2ee.jalonsod.ampa.business.AS;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;

import edu.uoc.pfc.j2ee.jalonsod.ampa.beans.Alumno;
import edu.uoc.pfc.j2ee.jalonsod.ampa.beans.Padre;
import edu.uoc.pfc.j2ee.jalonsod.ampa.beans.Socio;
import edu.uoc.pfc.j2ee.jalonsod.ampa.beans.SolicitudAlumno;
import edu.uoc.pfc.j2ee.jalonsod.ampa.beans.SolicitudPadre;
import edu.uoc.pfc.j2ee.jalonsod.ampa.beans.SolicitudSocio;
import edu.uoc.pfc.j2ee.jalonsod.ampa.business.ConstantesNegocio;
import edu.uoc.pfc.j2ee.jalonsod.ampa.business.BO.BusinessObject;
import edu.uoc.pfc.j2ee.jalonsod.ampa.business.BO.SocioBO;
import edu.uoc.pfc.j2ee.jalonsod.ampa.business.BO.SolicitudSocioBO;
import edu.uoc.pfc.j2ee.jalonsod.mtp.MTPUtils;

/**
 * File: SolicitudAltaAS.java
 * Created: 2012-05-25
 * 
 * Fachada de la capa de negocio para funciones relacionadas con la solicitud de alta de socios.
 * 
 * @author Jos Alonso de Motta
 */
public class SolicitudAltaAS extends BusinessObject{
	private final static Logger logger = Logger.getLogger(MTPUtils.getPackageName());
	
	/**
	 * Anula (cancela) una solicitud para hacerse socio.
	 * 
	 * @param usuario Identificador de socio de la solicitud.
	 * @return        true si la operacin tiene xito, false en caso contrario.
	 */
	public boolean anularSolicitud(String usuario) {
		try {
			SolicitudSocioBO solicitudSocioBO = new SolicitudSocioBO();
			session.getTransaction().begin();
			SolicitudSocio solicitudSocio = solicitudSocioBO.getSolicitudSocioPorUsuario(usuario);
			if(solicitudSocio!=null){
				solicitudSocioBO.eliminarSolicitudSocio(solicitudSocio);
			}else{
				throw new Exception("Solicitud no encontrada");
			}
			session.getTransaction().commit();
			return true;
		} catch (Exception e) {
			logger.warning("Error anulando solicitud de socio. Se hace rollback.");
			e.printStackTrace();
			session.getTransaction().rollback();
			return false;
		}
	}

	/**
	 * Crea una nueva solicitud para hacerse socio.
	 * 
	 * @param solicitudSocio Solicitud de socio a registrar.
	 * @return               true si la operacin tiene xito, false en caso contrario.
	 */
	public boolean crearSolicitudSocio(SolicitudSocio solicitudSocio) {
		try {
			SolicitudSocioBO solicitudSocioBO = new SolicitudSocioBO();
			session.getTransaction().begin();
			solicitudSocioBO.crearSolicitudSocio(solicitudSocio);
			session.getTransaction().commit();
			return true;
		} catch (Exception e) {
			logger.warning("Error insertando solicitud de socio. Se hace rollback.");
			e.printStackTrace();
			session.getTransaction().rollback();
			return false;
		}
	}

	/**
	 * Obtiene el nmero de solicitudes de socios pendientes de atender.
	 * 
	 * @return Nmero de solicitudes de socios pendientes de atender, o -1 en caso de error.
	 */
	public int getNumSolicitudesPendientes(){
		try {
			SolicitudSocioBO solicitudSocioBO = new SolicitudSocioBO();
			session.getTransaction().begin();
			int numSolicitudesPendientes = solicitudSocioBO.getNumSolicitudesPendientes();
			session.getTransaction().commit();
			return numSolicitudesPendientes;
		} catch (Exception e) {
			logger.warning("Error consultando nmero de solicitudes de socio. Se hace rollback.");
			e.printStackTrace();
			session.getTransaction().rollback();
			return -1;
		}
	}
	
	/**
	 * Obtiene todas las solicitudes para hacerse socio. Se fuerza al motor de persistencia a obtener
	 * los datos de padres y alumnos para su lectura posterior sin conexin a la base de datos.
	 * 
	 * @return Lista de solicitudes para hacerse socio, o null en caso de error.
	 */
	public List<SolicitudSocio> getSolicitudesSocio() {
		try {
			SolicitudSocioBO solicitudSocioBO = new SolicitudSocioBO();
			session.getTransaction().begin();
			List<SolicitudSocio> solicitudesSocio = solicitudSocioBO.getSolicitudesSocio();
			Iterator<SolicitudSocio> itSS = solicitudesSocio.iterator();
			while(itSS.hasNext()){
				SolicitudSocio ss = itSS.next();
				// Forzamos a leer los datos de padres y alumnos
				ss.getPadres().size();
				ss.getAlumnos().size();
			}
			session.getTransaction().commit();
			return solicitudesSocio;
		} catch (Exception e) {
			logger.warning("Error consultando nmero de solicitudes de socio. Se hace rollback.");
			e.printStackTrace();
			session.getTransaction().rollback();
			return null;
		}
	}

	/**
	 * Crea un nuevo socio, junto con sus padres y alumnos en base a una solicitud para hacerse socio.
	 * Despus elimina la solicitud.
	 * 
	 * @param usuario Identificador de socio de la solicitud.
	 * @return        true si la operacin tiene xito, false en caso contrario.
	 */
	public boolean validarSolicitud(String usuario) {
		try {
			SolicitudSocioBO solicitudSocioBO = new SolicitudSocioBO();
			SocioBO socioBO = new SocioBO();
			session.getTransaction().begin();
			SolicitudSocio solicitudSocio = solicitudSocioBO.getSolicitudSocioPorUsuario(usuario);
			if(solicitudSocio!=null){
				List<Padre> padres = new ArrayList<Padre>();
				List<Alumno> alumnos = new ArrayList<Alumno>();
				Iterator<SolicitudPadre> itPadres = solicitudSocio.getPadres().iterator();
				while(itPadres.hasNext()){
					SolicitudPadre solicitudPadre = itPadres.next();
					padres.add(new Padre(solicitudSocio.getUsuario(), solicitudPadre.getNombre(),
							solicitudPadre.getApellidos(), solicitudPadre.getDocumento(),
							solicitudPadre.getEmail(), solicitudPadre.getTelefonos()));
				}
				Iterator<SolicitudAlumno> itAlumnos = solicitudSocio.getAlumnos().iterator();
				while(itAlumnos.hasNext()){
					SolicitudAlumno solicitudAlumno = itAlumnos.next();
					alumnos.add(new Alumno(solicitudSocio.getUsuario(), solicitudAlumno.getNombre(),
							solicitudAlumno.getApellidos(), solicitudAlumno.getCurso(),
							solicitudAlumno.getFechaNacimiento()));
				}
				socioBO.crearSocio(new Socio(solicitudSocio.getUsuario(), solicitudSocio.getPassword(),
						solicitudSocio.getDireccion(),
						solicitudSocio.getFechaSolicitud(), ConstantesNegocio.SOCIO_VALIDO_SI,
						padres, alumnos));
				solicitudSocioBO.eliminarSolicitudSocio(solicitudSocio);
				session.getTransaction().commit();
				return true;
			}else{
				throw new Exception("Solicitud no encontrada");
			}
		} catch (Exception e) {
			logger.warning("Error validando solicitud de socio. Se hace rollback.");
			e.printStackTrace();
			session.getTransaction().rollback();
			return false;
		}
	}
}
