package edu.uoc.pfc.j2ee.jalonsod.ampa.business.BO;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.hibernate.Query;

import edu.uoc.pfc.j2ee.jalonsod.ampa.beans.Alumno;
import edu.uoc.pfc.j2ee.jalonsod.ampa.beans.AlumnoActividad;
import edu.uoc.pfc.j2ee.jalonsod.ampa.beans.AlumnoActividadPeriodo;
import edu.uoc.pfc.j2ee.jalonsod.ampa.beans.Padre;
import edu.uoc.pfc.j2ee.jalonsod.ampa.beans.PadreActividad;
import edu.uoc.pfc.j2ee.jalonsod.ampa.beans.PadreActividadPeriodo;
import edu.uoc.pfc.j2ee.jalonsod.ampa.beans.Socio;

/**
 * File: SocioBO.java
 * Created: 2012-06-06
 * 
 * Objeto de negocio para las operaciones relacionadas con socios.  
 * 
 * @author Jos Alonso de Motta
 */
public class SocioBO extends BusinessObject {
	
	/**
	 * Actualiza (update) en la base de datos un socio.
	 * 
	 * @param socio Socio con los datos actuales.
	 */
	public void actualizarSocio(Socio socio){
		session.update(socio);
	}
	
	/**
	 * Registra en la base de datos una nueva incscripcin de un alumno a una actividad.
	 * 
	 * @param alumnoActividad Nueva inscripcin a registrar.
	 */
	public void crearAlumnoActividad(AlumnoActividad alumnoActividad) {
		session.persist(alumnoActividad);
	}

	/**
	 * Registra en la base de datos una nueva incscripcin de un alumno a un periodo de una actividad.
	 * 
	 * @param aap Nueva inscripcin a un periodo a registrar.
	 */
	public void crearAlumnoActividadPeriodo(AlumnoActividadPeriodo aap){
		session.persist(aap);
	}

	/**
	 * Registra en la base de datos una nueva incscripcin de un padre a una actividad.
	 * 
	 * @param padreActividad Nueva inscripcin a registrar.
	 */
	public void crearPadreActividad(PadreActividad padreActividad) {
		session.persist(padreActividad);
	}
	
	/**
	 * Registra en la base de datos una nueva incscripcin de un padre a un periodo de una actividad.
	 * 
	 * @param pap Nueva inscripcin a un periodo a registrar.
	 */
	public void crearPadreActividadPeriodo(PadreActividadPeriodo pap){
		session.persist(pap);
	}

	/**
	 * Registra en la base de datos un nuevo socio.
	 * 
	 * @param socio Nuevo socio a registrar.
	 */
	public void crearSocio(Socio socio){
		session.persist(socio);
	}

	/**
	 * Elimina de la base de datos una inscripcin de un alumno a una actividad.
	 * 
	 * @param alumnoActividad Inscripcin del alumno a una actividad. 
	 */
	public void eliminarAlumnoActividad(AlumnoActividad alumnoActividad){
		session.delete(alumnoActividad);
	}

	/**
	 * Elimina de la base de datos una inscripcin de un padre a una actividad.
	 * 
	 * @param alumnoActividad Inscripcin del padre a una actividad. 
	 */
	public void eliminarPadreActividad(PadreActividad padreActividad){
		session.delete(padreActividad);
	}

	/**
	 * Elimina de la base de datos a un socio, incluyendo todos los datos relacionados.
	 * 
	 * @param socio Socio a eliminar de la base de datos.
	 */
	public void eliminarSocio(Socio socio){
		session.delete(socio);
	}

	/**
	 * Obtiene la lista de inscripciones a actividades de un alumno.
	 * 
	 * @param alumno Alumno del que se van a obtener las actividades inscritas.
	 * @return       Lista de actividades inscritas por el alumno.
	 */
	public List<AlumnoActividad> getActividadesAlumno(Alumno alumno) {
		Query query = session.createQuery("from AlumnoActividad where idAlumno=:idAlumno")
				.setParameter("idAlumno", alumno.getIdAlumno());
		@SuppressWarnings("unchecked")
		List<AlumnoActividad> actividadesAlumno = query.list();
		return actividadesAlumno;
	}

	/**
	 * Obtiene la lista de inscripciones a actividades de todos los alumnos de un socio.
	 * 
	 * @param socio Socio del que se quieren obtener las inscripciones de sus alumnos.
	 * @return      Lista de actividades inscritas por los alumnos del socio.
	 */
	public List<AlumnoActividad> getActividadesAlumnoPorSocio(Socio socio) {
		Iterator<Alumno> itAlumnos = getAlumnos(socio).iterator();
		List<AlumnoActividad> actividadesAlumno = new ArrayList<AlumnoActividad>();
		while(itAlumnos.hasNext()){
			Alumno alumno = itAlumnos.next();
			actividadesAlumno.addAll(getActividadesAlumno(alumno));
		}
		return actividadesAlumno;
	}

	/**
	 * Obtiene la lista de inscripciones a actividades de un padre.
	 * 
	 * @param padre  Padre del que se van a obtener las actividades inscritas.
	 * @return       Lista de actividades inscritas por el padre.
	 */
	public List<PadreActividad> getActividadesPadre(Padre padre) {
		Query query = session.createQuery("from PadreActividad where idPadre=:idPadre")
				.setParameter("idPadre", padre.getIdPadre());
		@SuppressWarnings("unchecked")
		List<PadreActividad> actividadesPadre = query.list();
		return actividadesPadre;
	}

	/**
	 * Obtiene la lista de inscripciones a actividades de todos los padres de un socio.
	 * 
	 * @param socio Socio del que se quieren obtener las inscripciones de sus padres.
	 * @return      Lista de actividades inscritas por los padres del socio.
	 */
	public List<PadreActividad> getActividadesPadrePorSocio(Socio socio) {
		Iterator<Padre> itPadres = getPadres(socio).iterator();
		List<PadreActividad> actividadesPadre = new ArrayList<PadreActividad>();
		while(itPadres.hasNext()){
			Padre padre = itPadres.next();
			actividadesPadre.addAll(getActividadesPadre(padre));
		}
		return actividadesPadre;
	}

	/**
	 * Carga de la base de datos un alumno en base a su identificador y el de su socio.
	 * 
	 * @param usuario  Identificador del socio.
	 * @param idAlumno Identificador del alumno.
	 * @return         Alumno obtenido o null si no lo encuentra.
	 */
	public Alumno getAlumno(String usuario, Integer idAlumno) {
		Query query = session.createQuery("from Alumno where usuario=:usuario and idAlumno=:idAlumno")
				.setParameter("usuario", usuario)
				.setParameter("idAlumno", idAlumno);
		@SuppressWarnings("unchecked")
		List<Alumno> alumnos = query.list();
		if(alumnos.size()==1){
			return alumnos.get(0);
		}else{
			return null;
		}
	}

	/**
	 * Carga de la base de datos una inscripcin a una actividad de un alumno en base a su clave.
	 * 
	 * @param idAlumno    Identificador del alumno.
	 * @param idActividad Identificador de la actividad.
	 * @return            Inscripcin a una actividad de un alumno.
	 */
	public AlumnoActividad getAlumnoActividad(Integer idAlumno, Integer idActividad) {
		Query query = session.createQuery("from AlumnoActividad where idAlumno=:idAlumno and idActividad=:idActividad")
				.setParameter("idAlumno", idAlumno)
				.setParameter("idActividad", idActividad);
		@SuppressWarnings("unchecked")
		List<AlumnoActividad> alumnosActividades = query.list();
		if(alumnosActividades.size()==1){
			return alumnosActividades.get(0);
		}else{
			return null;
		}
	}

	/**
	 * Carga de la base de datos los alumnos de un socio.
	 * 
	 * @param socio Socio del que se quieren recuperar los alumnos.
	 * @return      Lista de alumnos del socio.
	 */
	public List<Alumno> getAlumnos(Socio socio) {
		Query query = session.createQuery("from Alumno where usuario=:usuario")
				.setParameter("usuario", socio.getUsuario());
		@SuppressWarnings("unchecked")
		List<Alumno> alumnos = query.list();
		return alumnos;
	}

	/**
	 * Carga de la base de datos un padre en base a su identificador y el de su socio.
	 * 
	 * @param usuario  Identificador del socio.
	 * @param idPadre  Identificador del padre.
	 * @return         Padre obtenido o null si no lo encuentra.
	 */
	public Padre getPadre(String usuario, Integer idPadre) {
		Query query = session.createQuery("from Padre where usuario=:usuario and idPadre=:idPadre")
				.setParameter("usuario", usuario)
				.setParameter("idPadre", idPadre);
		@SuppressWarnings("unchecked")
		List<Padre> padres = query.list();
		if(padres.size()==1){
			return padres.get(0);
		}else{
			return null;
		}
	}

	/**
	 * Carga de la base de datos una inscripcin a una actividad de un padre en base a su clave.
	 * 
	 * @param idPadre     Identificador del padre.
	 * @param idActividad Identificador de la actividad.
	 * @return            Inscripcin a una actividad de un padre.
	 */
	public PadreActividad getPadreActividad(Integer idPadre, Integer idActividad) {
		Query query = session.createQuery("from PadreActividad where idPadre=:idPadre and idActividad=:idActividad")
				.setParameter("idPadre", idPadre)
				.setParameter("idActividad", idActividad);
		@SuppressWarnings("unchecked")
		List<PadreActividad> padresActividades = query.list();
		if(padresActividades.size()==1){
			return padresActividades.get(0);
		}else{
			return null;
		}
	}

	/**
	 * Carga de la base de datos los padres de un socio.
	 * 
	 * @param socio Socio del que se quieren recuperar los padres.
	 * @return      Lista de padres del socio.
	 */
	public List<Padre> getPadres(Socio socio) {
		Query query = session.createQuery("from Padre where usuario=:usuario")
				.setParameter("usuario", socio.getUsuario());
		@SuppressWarnings("unchecked")
		List<Padre> padres = query.list();
		return padres;
	}

	/**
	 * Carga de la base de datos un socio en base a su identificador.
	 * 
	 * @param usuario Identificador del socio a recuperar.
	 * @return        Socio encontrado.
	 */
	public Socio getSocio(String usuario){
		return (Socio)session.get(Socio.class, usuario);
	}

	/**
	 * Carga de la base de datos todos los socios.
	 * 
	 * @return Lista de socios.
	 */
	public List<Socio> getSocios() {
		Query query = session.createQuery("from Socio");
		@SuppressWarnings("unchecked")
		List<Socio> socios = query.list();
		return socios;
	}
	
	/**
	 * marca como invlidos a todos los socios.
	 *  
	 * @return nmero de socios invalidados.
	 */
	public int invalidarSocios(){
		Query query = session.createQuery("update Socio set valido='N'");
		int filasAfectadas = query.executeUpdate();
		return filasAfectadas;
	}
}
