package edu.uoc.pfc.j2ee.jalonsod.mtp;

import java.io.IOException;
import java.util.logging.Logger;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import edu.uoc.pfc.j2ee.jalonsod.mtp.config.Configurator;
import edu.uoc.pfc.j2ee.jalonsod.mtp.config.ConfiguratorException;
import edu.uoc.pfc.j2ee.jalonsod.mtp.helper.AccessControl;
import edu.uoc.pfc.j2ee.jalonsod.mtp.helper.ContextFactory;
import edu.uoc.pfc.j2ee.jalonsod.mtp.helper.ModelMapper;
import edu.uoc.pfc.j2ee.jalonsod.mtp.helper.ModelRunner;
import edu.uoc.pfc.j2ee.jalonsod.mtp.helper.Validator;
import edu.uoc.pfc.j2ee.jalonsod.mtp.helper.ViewMapper;
import edu.uoc.pfc.j2ee.jalonsod.mtp.request.RequestData;
import edu.uoc.pfc.j2ee.jalonsod.mtp.request.RequestDataPrivate;

/**
 * File: Controller.java
 * Created: 2012-04-21
 * 
 * The front controller of the framework. Loads configuration & controls the request life cycle.  
 * 
 * @author Jos Alonso de Motta
 */
public class Controller extends HttpServlet {
	private final static Logger logger = Logger.getLogger(RequestData.MTP_LOGGER);
	private static final long serialVersionUID = 1L;
	/**
	 * The initialization parameter name in the web.xml that holds the application config file name
	 * for the framework. 
	 */
	private static final String PARAMETER_CONFIGFILE = "configFile";

	/**
	 * The initialization parameter name in the web.xml that holds the application logger level.
	 */
	private static final String PARAMETER_LOGGER_LEVEL= "loggerLevel";
    
    public Controller() {
        super();
    }

    /**
     * Loads configuration in the initialization stage of the servlet
     */
	public void init(ServletConfig config) throws ServletException {
		MTPUtils.setLoggerToLevel(config.getInitParameter(PARAMETER_LOGGER_LEVEL));
		logger.entering(MTPUtils.getClassName(), MTPUtils.getMethodName(), config);
		try {
			Configurator.getInstance(config.getInitParameter(Controller.PARAMETER_CONFIGFILE));
			logger.info("Configuration loaded successfully");
		} catch (ConfiguratorException e) {
			e.printStackTrace();
			logger.severe("Error loading configuration in Controller.init");
			throw new ServletException("Error loading configuration in Controller.init", e);
		}
		logger.exiting(MTPUtils.getClassName(), MTPUtils.getMethodName(), config);
	}

	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		// All get & post request are processed
		doGetPost(request, response);
	}

	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		// All get & post request are processed
		doGetPost(request, response);
	}

	/**
	 * Request life cycle processor.
	 * 
	 * @param request the <code>request</code> object.
	 * @param response the <code>response</code> object
	 * @throws ServletException
	 * @throws IOException
	 */
	protected void doGetPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		logger.entering(MTPUtils.getClassName(), MTPUtils.getMethodName());
		logger.fine("Controller. Request requested!");
		MTPException mtpException = null;
		RequestDataPrivate requestData = null;
		try {
			// Step 1: load context (& create session on first user request)
			try {
				logger.finest("Controller. Step 1. ContextCreate");
				requestData = ContextFactory.getContext(request);
				logger.finest("Controller. Step 1. ContextCreate. Request data " + ((requestData!=null)?"is loaded":"is not loaded"));
			} catch (Exception e) {
				throw new MTPException("Error recovering requestData", e, MTPException.MTP_EXCEPTION_REQUEST_DATA);
			}
			
			// Step 2: map model (& create model Instance)
			ModelMapper modelMapper;
			try {
				logger.finest("Controller. Step 2. ModelMapper");
				modelMapper = requestData.getConfigurator().getModelMapperClassReference().newInstance();
				modelMapper.setRequestModel(requestData);
				logger.finest("Controller. Step 2. ModelMapper. Model selected is " + requestData.getModel().getName());
			} catch (Exception e) {
				throw new MTPException("Error instantiating ModelMapper", e, MTPException.MTP_EXCEPTION_MODEL_MAPPER);
			}
			
			// Step 3: security check
			try {
				logger.finest("Controller. Step 3. AccessControl");
				AccessControl accessControl = new AccessControl();
				accessControl.securityCheck(requestData);
				logger.finest("Controller. Step 3. AccessControl. Passed? " + (requestData.isAuthorized()?"yes":"no"));
			} catch (Exception e){
				throw new MTPException("Error while checking security.", e, MTPException.MTP_EXCEPTION_SECURITY_CHECK);
			}
			
			if(requestData.isAuthorized()){
				// Step 4: validation
				try {
					logger.finest("Controller. Step 4. Validator");
					Validator validator = new Validator();
					validator.validateRequest(requestData);
					logger.finest("Controller. Step 4. Validator. Passed? " + (requestData.isValidated()?"yes":("no. Stop in step " + (requestData.getValidationStatus()))));
				} catch (Exception e){
					e.printStackTrace();
					throw new MTPException("Error while validating source data.", MTPException.MTP_EXCEPTION_VALIDATION);
				}
			}

			if(requestData.isValidated()){
				// Step 5: run model
				try {
					logger.finest("Controller. Step 5. RunModel. Model to run: " + requestData.getModelInstance().getClass().getName());
					ModelRunner modelRunner = new ModelRunner();
					modelRunner.run(requestData);
					logger.finest("Controller. Step 5. RunModel. Model returned " + requestData.getModelResponse());
				} catch (Exception e) {
					throw new MTPException("Error running model", e, MTPException.MTP_EXCEPTION_MODEL_RUN);
				}
			}

			// Step 6: map view
			ViewMapper viewMapper;
			try {
				logger.finest("Controller. Step 6. ViewMapper");
				viewMapper = requestData.getConfigurator().getViewMapperClassReference().newInstance();
				viewMapper.setRequestView(requestData);
				logger.finest("Controller. Step 6. ViewMapper. Returned view " + requestData.getView());
			} catch (Exception e) {
				throw new MTPException("Error mapping view", e, MTPException.MTP_EXCEPTION_VIEW_MAPPER);
			}

			// Step 7: dispatch view
			try {
				logger.finest("Controller. Step 7. DispatchView. Start forward to " + requestData.getView());
				request.getRequestDispatcher(requestData.getView()).forward(request, response);
				logger.finest("Controller. Step 7. DispatchView. Terminated forward");
			} catch (Exception e) {
				throw new MTPException("Error dispatching view", e, MTPException.MTP_EXCEPTION_VIEW_DISPATCHER);
			}
		} catch (MTPException e) {
			e.printStackTrace();
			logger.fine("Thrown exception in request");
			mtpException = e;
		}
		if(mtpException!=null){
			mtpException.printStackTrace();
			logger.warning("MTPException. errorCode: " + mtpException.getErrorCode() + ". Message: "
					+ mtpException.getMessage());
			if(requestData!=null){
				// Copy exception into request for error processing in default error view
				requestData.setRequestAttribute(RequestDataPrivate.REQUEST_EXCEPTION, mtpException);
				request.getRequestDispatcher(requestData.getConfigurator().getDefaultErrorView()).forward(request, response);
			}else{
				logger.severe("MTP Controller Servlet exception has no requestData");
				throw new ServletException("Error in MTP Framework configuration.", mtpException);
			}
		}
		logger.exiting(MTPUtils.getClassName(), MTPUtils.getMethodName());
	}
}
