package edu.uoc.pfc.j2ee.jalonsod.mtp.request;

import edu.uoc.pfc.j2ee.jalonsod.mtp.MTPException;
import edu.uoc.pfc.j2ee.jalonsod.mtp.config.Configurator;
import edu.uoc.pfc.j2ee.jalonsod.mtp.config.Parameter;


/**
 * File: ParameterInstance.java
 * Created: 2012-05-01
 * 
 * Holds one parameter value from the request. The <code>parameterInstance</code> is related to one
 * <code>parameter</code> metadata from configuration.
 * 
 * @author Jos Alonso de Motta
 * @see FormInstance
 * @see Parameter
*/
public abstract class ParameterInstance {
	// Parameter validation possible status.
	/**
	 * The parameter is not yet validated.
	 */
	public static final Integer VALIDATION_NOT_VALIDATED = 0;
	
	/**
	 * The parameter does not pass datatype conversion validation for the value.
	 */
	public static final Integer VALIDATION_DATATYPE_CONVERSION_ERROR = 1;
	
	/**
	 * The parameter value is null but it's required.
	 */
	public static final Integer VALIDATION_REQUIRED_VALUE_IS_NULL = 2;
	
	/**
	 * The parameter value does not pass function validation.
	 */
	public static final Integer VALIDATION_FUNCTION_ERROR = 3;
	
	/**
	 * The parameter is fully validated.
	 */
	public static final Integer VALIDATION_OK = 4;
	
	/**
	 * The configuration metadata <code>parameter</code> wich belongs to this parameterInstance.
	 */
	protected Parameter parameter;
	
	/**
	 * The validation status of the parameterInstance. It's one of the VALIDATION_* constants.
	 */
	protected Integer validationStatus;
	
	
	protected ParameterInstance(Parameter parameter){
		this.parameter = parameter;
	}
	
	/**
	 * Factory method to create new ParameterInstance subtypes.
	 * 
	 * @param parameter the configuration metadata <code>parameter</code> associated with this object.
	 * @return a new <code>parameterInstance</code> object of appropriate subtype.
	 */
	public static ParameterInstance createParameterInstance(Parameter parameter){
		if(parameter.getDatatype().equalsIgnoreCase(Configurator.DATATYPE_STRING)){
			return new StringParameterInstance(parameter);
		}else if (parameter.getDatatype().equalsIgnoreCase(Configurator.DATATYPE_BOOLEAN)){
			return new BooleanParameterInstance(parameter);
		}else if (parameter.getDatatype().equalsIgnoreCase(Configurator.DATATYPE_INTEGER)){
			return new IntegerParameterInstance(parameter);
		}else if (parameter.getDatatype().equalsIgnoreCase(Configurator.DATATYPE_DOUBLE)){
			return new DoubleParameterInstance(parameter);
		}else if (parameter.getDatatype().equalsIgnoreCase(Configurator.DATATYPE_DATE)){
			return new DateParameterInstance(parameter);
		}
		return null;
	}
	
	/**
	 * Sets the value of this <code>parameterInstance</code> with the default value defined in
	 * the <code>parameter</code> metadata from configuration.
	 */
	public void setValueAsDefault() {
		setValue(getDefaultValue());
	}

	// getters & setters
	public Parameter getParameter() {
		return parameter;
	}
	public void setParameter(Parameter parameter) {
		this.parameter = parameter;
	}
	public Integer getValidationStatus() {
		return validationStatus;
	}
	public void setValidationStatus(Integer validationStatus) {
		this.validationStatus = validationStatus;
	}

	/**
	 * Returns the value of the <code>parameterInstance</code>.
	 * 
	 * @return the value of the <code>parameterInstance</code>. The datatype varies according to
	 *         <code>parameter</code> subtype from configuration.
	 */
	public abstract Object getValue();
	
	/**
	 * Sets the value of the <code>parameterInstance</code>.
	 * 
	 * @param value the value of the <code>parameterInstance</code>. The datatype must be appropiate
	 *              according datatype of <code>parameter</code> metadata from configuration.
	 */
	public abstract void setValue(Object value);
	
	/**
	 * Sets the value of the <code>parameterInstance</code> received in a string and converted to
	 * appropriate datatype.
	 * 
	 * @param value         the string with the value to convert.
	 * @throws MTPException throws when the value cannot be converted to appropriate datatype.
	 */
	public abstract void setStringValue(String value) throws MTPException;
	
	/**
	 * Returns the default value of the parameterInstance with appropriate datatype.
	 * 
	 * @return the default value of the parameterInstance.
	 */
	public abstract Object getDefaultValue();
	
	public abstract String toString();
}
