package edu.uoc.pfc.j2ee.jalonsod.mtp.request;

import java.util.Hashtable;

import edu.uoc.pfc.j2ee.jalonsod.mtp.config.Configurator;
import edu.uoc.pfc.j2ee.jalonsod.mtp.config.Language;
import edu.uoc.pfc.j2ee.jalonsod.mtp.config.LanguageEntry;
import edu.uoc.pfc.j2ee.jalonsod.mtp.config.Model;
import edu.uoc.pfc.j2ee.jalonsod.mtp.config.Result;
import edu.uoc.pfc.j2ee.jalonsod.mtp.config.Role;


/**
 * File: RequestData.java
 * Created: 2012-04-29
 * 
 * RequestData is the faade API for applications using the framework.
 * It's also the store of the MTP, session and request information during the request life cycle.
 * <p>
 * Applications using the framework must use this API to interact with the framework.
 * 
 * The life of an requestData object is the same as the request.
 * 
 * @author Jos Alonso de Motta
*/
public interface RequestData {
	/**
	 * Logger name of the framework.
	 */
	public static final String MTP_LOGGER = "edu.uoc.pfc.j2ee.jalonsod.mtp";

	// Attributes used to store data in the request/session.
	/**
	 * The request attribute name of the RequestData object.
	 */
	public static final String REQUEST_REQUESTDATA_ATTRIBUTE = "MTPRequestData";
	
	/**
	 * The request attribute name of the exception thrown in the request life-cycle. If there is no
	 * exception thrown, the attribute doesn't exists.
	 */
	public static final String REQUEST_EXCEPTION = "MTPException";

	
	// Validation stages
	/**
	 * Request has not been validated, because no validation is done.
	 * 
	 * @see RequestData#getValidationStatus()
	 */
	public static final int VALIDATION_NO_VALIDATED = 0;
	
	/**
	 * Request has not been validated, because datatype conversion has failed.
	 * 
	 * @see RequestData#getValidationStatus()
	 */
	public static final int VALIDATION_ERROR_STAGE_1_DATATYPE_CONVERSION = 1;

	/**
	 * Request has not been validated, because some required fields are empty.
	 * 
	 * @see RequestData#getValidationStatus()
	 */
	public static final int VALIDATION_ERROR_STAGE_2_REQUIRED_FIELDS = 2;

	/**
	 * Request has not been validated, because some field validation functions has returned with error.
	 * 
	 * @see RequestData#getValidationStatus()
	 */
	public static final int VALIDATION_ERROR_STAGE_3_FIELD_VALIDATION_FUNCTIONS = 3;

	/**
	 * Request has not been validated, because some fields cannot be loaded into application model object.
	 * 
	 * @see RequestData#getValidationStatus()
	 */
	public static final int VALIDATION_ERROR_LOADING_PARAMETER_VALUES_INTO_MODEL = 4;

	/**
	 * Request has not been validated, because the validation function of the model has returned with error.
	 * 
	 * @see RequestData#getValidationStatus()
	 */
	public static final int VALIDATION_ERROR_STAGE_4_MODEL_VALIDATION_FUNCTION = 5;
	
	/**
	 * Request has been validated, all stages are passed right.
	 * 
	 * @see RequestData#getValidationStatus()
	 */
	public static final int VALIDATION_OK = 6;


	// Configuration data
	/**
	 * Returns the main configuration object.
	 * 
	 * @return the configuration main object.
	 * @see    Configurator
	 */
	public Configurator getConfigurator();

	/**
	 * Returns the application supported languages.
	 * 
	 * @return the list of application supported languages.
	 * @see    Language
	 */
	public Hashtable<String, Language> getLanguages();

	
	// Session data
	/**
	 * Creates a new session for the user (framework creates a new session for the request if it does
	 * not has one).
	 * <p>
	 * This is done automatically by the framework, but application can create a new session for the
	 * request.
	 *  
	 * @see RequestData#hasValidSession()
	 * @see RequestData#logoutSession()
	 */
	public void createSession();
	
	/**
	 * Checks if the request has a valid session.
	 * 
	 * @return <code>true</code> if request has a valid session, <code>false</code> otherwise.
	 * @see    RequestData#createSession()
	 * @see    RequestData#logoutSession()
	 */
	public boolean hasValidSession();
	
	/**
	 * Invalidates the session of the request.
	 * 
	 * @see RequestData#createSession()
	 * @see RequestData#hasValidSession()
	 */
	public void logoutSession();
	
	//// Session Attributes
	/**
	 * Returns an attribute stored previously in the session.
	 * 
	 * @param attributeName the name of the attribute to obtain.
	 * @return              the object stored in the session, or  <code>null</code> if it's empty or
	 *                      doesn't exists.
	 */
	public Object getSessionAttribute(String attributeName);
	
	/**
	 * Stores an attribute into the session.
	 * 
	 * @param attributeName   the name of the attribute to store.
	 * @param attributeObject the object to store in the session.
	 */
	public void setSessionAttribute(String attributeName, Object attributeObject);
	
	/**
	 * Removes an attribute from the session.
	 * 
	 * @param attributeName the name of the attribute to remove.
	 */
	public void deleteSessionAttribute(String attributeName);
	
	
	//// Session Roles
	/**
	 * Gets the roles granted to the session.
	 * 
	 * @return the list of roles granted to the session.
	 * @see    Role
	 */
	public Hashtable<String, Role> getRoles();
	
	/**
	 * Sets the roles granted to the session.
	 * 
	 * @param sessionRoles the list of roles to grant to the session. The previous granted roles
	 *                     are replaced with these.
	 * @see                Role
	 */
	public void setRoles(Hashtable<String, Role> sessionRoles);
	
	
	//// Internationalization
	/**
	 * Gets the <code>language</code> inferred for the request, based on previous
	 * <code>setUserLanguage</code> call, request agent supported languages or default application
	 * language.
	 *  
	 * @return the <code>language</code> inferred for the request.
	 * @see    Language
	 */
	public Language getUserLanguage();
	
	/**
	 * Sets the language for the session. This language is stored in the session for use on the next
	 * requests from the same session.
	 * 
	 * @param userLanguage the <code>language</code> to assign to the session.
	 * @see   Language
	 */
	public void setUserLanguage(Language userLanguage);
	
	/**
	 * Obtains a message text from a <code>languageEntry</code> in the request user's language with
	 * the supplied key.
	 * 
	 * @param key the key string to match with a <code>languageEntry</code> key. 
	 * @return    a <code>string</code> message in request inferred language or <code>null</code> if
	 *            not found.
	 * @see       Language
	 * @see       LanguageEntry
	 */
	public String getMessage(String key);

	// Request data
	//// Request attributes & parameters
	/**
	 * Returns an attribute stored previously in the request.
	 * 
	 * @param attributeName the name of the attribute to obtain.
	 * @return              the object stored in the request with supplied name, or
	 *                      <code>null</code> if it's empty or doesn't exists.
	 */
	public Object getRequestAttribute(String attributeName);
	
	/**
	 * Stores an attribute in the request. The life of the attribute is the same as the request.
	 * 
	 * @param attributeName   the name of the attribute to store.
	 * @param attributeObject the object to store in the request.
	 */
	public void setRequestAttribute(String attributeName, Object attributeObject);
	
	/**
	 * Obtains a parameter from the request.
	 * 
	 * @param parameterName the name of the parameter to obtain.
	 * @return              the <code>string</code> with the parameter value or <code>null</code> if
	 *                      the parameter doesn't exists.
	 */
	public String getRequestParameter(String parameterName);

	
	//// Framework generated data about request
	/**
	 * Gets the <code>model</code> configuration object selected for the request, or
	 * <code>null</code> if the request has no model.
	 * 
	 * @return the model configuration object.
	 * @see    Model
	 */
	public Model getModel();
	
	/**
	 * Returns the application model instance object.
	 * 
	 * @return the application model instance object.
	 */
	public Object getModelInstance();

	/**
	 * Returns the <code>formInstance</code> object with the parameters of the request associated
	 * with the <code>model</code>.
	 * 
	 * @return the <code>formInstance</code> configuration object.
	 * @see    FormInstance
	 */
	public FormInstance getFormInstance();
	
	/**
	 * Returns the response <code>string</code> returned by the run method of the application model
	 * object.
	 * 
	 * @return the response <code>string</code> returned by the run method of the application model
	 *         object.
	 */
	public String getModelResponse();
	
	/**
	 * Returns the <code>result</code> object of the request.
	 *  
	 * @return the <code>result</code> object of the request or <code>null</code> if it doesn't exists.
	 * @see    Model
	 * @see    Result
	 */
	public Result getResult();
	
	/**
	 * Returns the view name to which the response will be redirected.
	 * 
	 * @return the view name selected for the response.
	 */
	public String getView();
	
	/**
	 * Returns the request session authorization status to run the model.
	 * 
	 * @return <code>true</code> if request session has been authorized, or <code>false</code> otherwise.
	 */
	public boolean isAuthorized();
	
	/**
	 * Returns the request validation status.
	 * 
	 * @return the validation status of the request. It's one of the
	 *         <code>RequestData.VALIDATION_*</code> values. 
	 */
	public int getValidationStatus();
	
	/**
	 * Returns if the request has passed all validations.
	 * 
	 * @return <code>true</code> if the request has passed all validations, <code>false</code> otherwise.
	 */
	public boolean isValidated();
}
