package edu.uoc.pfc.j2ee.jalonsod.mtp.request;

import java.util.Hashtable;

import edu.uoc.pfc.j2ee.jalonsod.mtp.config.Language;
import edu.uoc.pfc.j2ee.jalonsod.mtp.config.Model;
import edu.uoc.pfc.j2ee.jalonsod.mtp.config.Result;
import edu.uoc.pfc.j2ee.jalonsod.mtp.helper.ModelMapper;
import edu.uoc.pfc.j2ee.jalonsod.mtp.helper.ViewMapper;


/**
 * File: RequestDataPrivate.java
 * Created: 2012-05-10
 * 
 * The API of the requestData private to components of MTP framework.
 * The protocol implementations of the requestData must implements both <code>RequestData</code> and
 * <code>RequestDataPrivate</code>.
 * 
 * @author Jos Alonso de Motta
 * @see    RequestData
 */
public interface RequestDataPrivate extends RequestData {

	// Private attributes used to store data in the request/session
	/**
	 * The session attribute name for roles granted to a session. Use <code>getRoles()</code> to
	 * read it and <code>setRoles()</code> to set.
	 * 
	 * @see RequestData#getRoles()
	 * @see RequestData#setRoles(Hashtable)
	 */
	public static final String SESSION_ROLES_ATTRIBUTE = "MTPSessionRoles";
	
	/**
	 * The session attribute name for the language of the session. Set with
	 * <code>setUserLanguage()</code>.
	 *  
	 * @see RequestData#getUserLanguage()
	 * @see RequestData#setUserLanguage(Language)
	 */
	public static final String SESSION_LANGUAGE_ATTRIBUTE = "MTPSessionLanguage";
	
	/**
	 * The request attribute name for the model instance object of application. Read with
	 * <code>getModelInstance</code> method.
	 *  
	 * @see RequestData#getModelInstance()
	 */
	
	public static final String REQUEST_MODEL_ATTRIBUTE = "MTPRequestModel";


	//// Framework generated data about request
	/**
	 * Sets the <code>model</code> configuration object for the request.
	 * <p>
	 * This is done by the <code>modelMapper</code>.
	 * 
	 * @param model the <code>model</code> configuration object to set into the request.
	 * @see         Model
	 */
	public void setModel(Model model);

	/**
	 * Sets the application model object.
	 * <p>
	 * This is done by the <code>modelMapper</code>.
	 * 
	 * @param modelInstance the application model object.
	 * @see   ModelMapper
	 */
	public void setModelInstance(Object modelInstance);

	/**
	 * Sets the <code>formInstance</code> object with the parameters of the request associated with
	 * the <code>model</code>.
	 * <p>
	 * This is done in the validation stage by the <code>validator</code>.
	 * 
	 * @param formInstance the formInstance object with the parameters of the request associated
	 *                     with the <code>model</code>.
	 */
	public void setFormInstance(FormInstance formInstance);
	
	/**
	 * Sets the response <code>string</code> returned by the run method of the application model
	 * object.
	 * <p>
	 * This is done by the framework when the model run method finishes.
	 * 
	 * @param modelResponse the response <code>string</code> returned by the run method of the
	 *                      application model object.
	 */
	public void setModelResponse(String modelResponse);
	
	/**
	 * Sets the result object of the request.
	 * <p>
	 * This is done by the <code>viewMapper</code>.
	 * 
	 * @param result the result object of the request.
	 * @see          ViewMapper
	 */
	public void setResult(Result result);
	
	/**
	 * Sets the view name to which response will be redirected.
	 * <p>
	 * This is done by the <code>viewMapper</code>.
	 * 
	 * @param view
	 */
	public void setView(String view);
	
	/**
	 * Sets the request authorization status to run the model.
	 * <p>
	 * This is done by the <code>accessControl</code> of the framework.
	 * 
	 * @param authorized <code>true</code> if request is authorized, <code>false</code> otherwise.
	 */
	public void authorize(boolean authorized);
	
	/**
	 * Sets the request validation status.
	 * <p>
	 * This is done by the framework in validation stage.
	 * 
	 * @param validationStatus the validation status of the framework. It's one of the 
	 *                         <code>RequestData.VALIDATION_*</code> values. 
	 */
	public void setValidationStatus(int validationStatus);
}
